package EasyFile;

import java.util.EventObject;

/**
 * Ereignis, das f&uuml;r jeden Verzeichniseintrag bei einem
 * Verzeichnis-Listing erzeugt. Das Verzeichnis-Listing kann
 * sich auf das aktuelle Verzeichnis des Ftp-Servers oder des
 * Ftp-Clients beziehen.
 * @author Henrik Warnecke, J&ouml;rg Hoos, Matthias Ansorg, Thomas Martin, Ralf Mei&szlig;ner
 * @version 0.3
 */

public class DirEntryEvent extends EventObject {

  public final static int isFtp=0;
  public final static int isLocal=1;

  /**
   * Gibt an, ob sich das Event auf ein Verzeichnis-Listing auf
   * dem Ftp-Server (Wert <code>isFtp</code>==0) oder dem lokalen
   * System (Wert <code>isLocal</code>==1) bezieht.
   */
  public int place;

  /**
   * Art des Verzeichniseintrags. Gibt an, ob der Verzeichniseintrag
   * ein Verzeichnis ist (Wert <code>true</code>) oder eine Datei
   * (Wert <code>false</code>).
   */
  public boolean isDir;

  /**
   * Name des Verzeichniseintrags. Ein Verzeichniseintrag ist ein
   * Verzeichnis oder eine Datei.
   */
  StringBuffer dirEntry= new StringBuffer();

  /**
   * Ein neues DirEntryEvent erzeugen.
   * @param  source  Objekt, von dem aus dieses DirEntryEvent
   * erzeugt wird. &Uuml;blicherweise &raquo;this&laquo;.
   */
  public DirEntryEvent(Object source) {
    super(source);
  }

  /**
   * Ermitteln des Wertes der Variablen dirEntry, die den Namen des mit diesem
   * {@link DirEntryEvent DirEntryEvent} &uuml;bermittelten Verzeichniseintrags
   * enth&auml;lt.
   * @return der Name des Verzeichniseintrags, der mit diesem
   * {@link DirEntryEvent DirEntryEvent} &uuml;bermittelt wird.
   */
  public String getDirEntry() {
    return dirEntry.toString();
  }

  /**
   * Setzen der Variablen dirEntry, die den Namen des mit diesem
   * {@link DirEntryEvent DirEntryEvent} &uuml;bermittelten Verzeichniseintrags
   * enth&auml;lt.
   * @param s der Wert, auf den der Name des Verzeichniseintrags gesetzt werden
   * soll. Er darf keine Verzeichnistrenner enthalten, sondern bezeichnet einen
   * Dateinmaen im aktuellen Verzeichnis.
   */
  public void setDirEntry(String s) {
    dirEntry.replace(0,dirEntry.length(),s);
  }
}