package EasyFile;

import javax.swing.UIManager;
import java.awt.*;

/**
 * Hauptklasse des Ftp-Clients EasyFile. Verantwortlich f&uuml;r
 * das Erzeugen des Hauptdialogs. Das Design ist so ausgelegt, dass
 * programmweit jeweils nur eine Instanz der Modellklassen (Mdl_*)
 * verwendet wird. Diese Instanzen werden im Konstruktor von
 * {@link EasyFileCtrl EasyFileCtrl} erzeugt und den GUI_Klassen
 * (GUI_*), die sie ben&ouml;tigen, als Parameter ihres Konstruktors
 * &uuml;bergeben.<br>
 * Verfahren, um EasyFile mit dieser Klasse au&szlig;erhalb von JBuilder zu
 * starten: man wechselt in das Wurzelverzeichnis des EasyFile-Verzeichnisbaums
 * und befiehlt etwa: <code>/opt/JBuilder6/jdk1.3.1/bin/java -classpath
 * ./classes/:/opt/JBuilder6/lib/beans/ipworks.jar EasyFile.EasyFileCtrl</code>
 *
 * @author Henrik Warnecke, J&ouml;rg Hoos, Matthias Ansorg, Thomas Martin, Rafl Mei&szlig;ner
 * @version 0.3
 */

public class EasyFileCtrl
{
  boolean packFrame = false;

/**
 * Den Hauptdialog des Ftp-Clients EasyFile erzeugen. Siehe
 * {@link GUI_HauptDlg GUI_HauptDlg} f&uuml;r die Quellen des
 * Hauptdialogs.
 */

  public EasyFileCtrl()
  {
    Mdl_Setting theSetting = new Mdl_Setting();
    Mdl_Log theLog = new Mdl_Log();
    Mdl_Hotlist theHotlist = new Mdl_Hotlist();
    Mdl_Ftp theFtp =
       new Mdl_Ftp(theHotlist, theLog, theSetting);
    GUI_HauptDlg frame =
       new GUI_HauptDlg(theFtp, theHotlist, theLog, theSetting);

    //Frames berprfen, die voreingestellte Gre haben
    //Frames packen, die nutzbare bevorzugte Greninformationen
    //enthalten, z.B. aus ihrem Layout
    if (packFrame)
    {
      frame.pack();
    }
    else
    {
      frame.validate();
    }

    //Das Fenster zentrieren
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    if (frameSize.height > screenSize.height)
    {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width)
    {
      frameSize.width = screenSize.width;
    }
    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    frame.setVisible(true);
  }

/**
 * Startet die Anwendung EasyFile.
 * @param args Kommandozeilenargumente f&uuml;r EasyFile. Es
 * werden keine Kommandozeilenargumente ausgewertet, so dass
 * der Wert dieser Variablen ignoriert wird.
 */

  public static void main(String[] args)
  {
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    new EasyFileCtrl();
  }
}