package EasyFile;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Dialog zur Einstellung von Optionen zur FTP-&Uuml;bertragung
 * durch EasyFile. Verantwortlich zur Darstellung der grafischen
 * Benutzeroberfl&auml;che f&uuml;r die Klasse {@link Mdl_Setting Mdl_Setting},
 * in der Proxyserver und &Uuml;bertragungsart eingestellt werden
 * k&ouml;nnen.
 * @author Ralf Mei&szlig;ner, Matthias Ansorg
 * @version 0.3
 */

public class GUI_EinstgDlg extends JDialog {
  Mdl_Setting theSetting;

  JPanel rootPanel = new JPanel();
  GridBagLayout gridBagLayout = new GridBagLayout();

  JLabel serverLabel = new JLabel();
  JTextField serverField = new JTextField();

  JLabel portLabel = new JLabel();
  JTextField portField = new JTextField();

  JLabel anonymLabel = new JLabel();

  JLabel userLabel = new JLabel();
  JTextField userField = new JTextField();

  JLabel passwordLabel = new JLabel();
  JTextField passwordField = new JTextField();

  JRadioButton asciiRadio = new JRadioButton();
  JRadioButton binaryRadio = new JRadioButton();
  ButtonGroup  transmissionModeGroup = new ButtonGroup();

  JButton okButton = new JButton();
  JButton cancelButton = new JButton();

  /**
   * Einstellungsdialog erzeugen.
   *
   * @param parent Fenster, von dem der Benutzer den
   * Einstellungsdialog aufgerufen hat.
   * @param theSetting anwendungsweit verwendete Einstellungen f&uuml;r die
   * Ftp-Verbindungen
   */
  public GUI_EinstgDlg(Frame parent, Mdl_Setting theSetting) {
    super(parent);
    this.theSetting = theSetting;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }

    //bisherige Einstellungen im Dialog anzeigen:
    serverField.setText(theSetting.getFwHost());
    portField.setText(Integer.toString(theSetting.fwPort));
    passwordField.setText(theSetting.getAnonymPassword());
    userField.setText(theSetting.getAnonymUser());
    if (theSetting.transfermode == theSetting.tmBinary)
      binaryRadio.setSelected(true);
    else
      asciiRadio.setSelected(true);
  }

  private void jbInit() throws Exception {
    this.setSize(new Dimension(400,250));
    this.setTitle("EasyFile - Einstellungen");
    this.getContentPane().add(rootPanel, BorderLayout.CENTER);

    rootPanel.setPreferredSize(new Dimension(400, 250));
    rootPanel.setLayout(gridBagLayout);

/*        GridBagConstraints(int gridx, int gridy,
                             int gridwidth, int gridheight,
                             double weightx, double weighty,
                             int anchor, int fill,
                             Insets insets(top,left,bottom,right), int ipadx, int ipady)
*/
    rootPanel.add(serverLabel,
      new GridBagConstraints(0, 0,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 0, 0), 0, 0));
    rootPanel.add(serverField,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 0,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(portLabel,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(portField,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 1,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(binaryRadio,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             2, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(asciiRadio,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             2, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(anonymLabel,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             2, 1,
                             1.0, 1.0,
                             GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(userLabel,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(userField,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 5,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(passwordLabel,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(passwordField,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 6,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(okButton,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.EAST, GridBagConstraints.NONE,
                             new Insets(0, 5, 5, 0), 0, 0));
    rootPanel.add(cancelButton,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 7,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 5), 0, 0));

    serverLabel.setText("Proxyserver");
    portLabel.setText("Port");
    transmissionModeGroup.add(binaryRadio);
    transmissionModeGroup.add(asciiRadio);
    binaryRadio.setText("Binaere Uebertragungsart");
    binaryRadio.setSelected(true);
    asciiRadio.setText("ASCII Uebertragungsart");
    anonymLabel.setText("Verwende fr anonymes Login:");
    userLabel.setText("Benutzername");
    passwordLabel.setText("Passwort");
    okButton.setText("OK");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        okButton_actionPerformed(e);
      }
    });
    cancelButton.setText("Abbrechen");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancelButton_actionPerformed(e);
      }
    });
  }

  /**
   * Dialog schlie&szlig;en ohne &Auml;nderungen zu speichern. Wird aufgerufen
   * &uuml;ber einen anonymen ActionListener beim Bet&auml;tigen der
   * Schaltfl&auml;che &raquo;Abbrechen&laquo;.
   * @param e Ausl&ouml;sendes &Auml;ktionevent
   */
  void cancelButton_actionPerformed(ActionEvent e) {
      dispose();
  }

  /**
   * Dialog schlie&szlig;en und &Auml;nderungen speichern. Wird aufgerufen
   * &uuml;ber einen anonymen ActionListener beim Bet&auml;tigen der
   * Schaltfl&auml;che &raquo;OK&laquo;.
   * @param e Ausl&ouml;sendes &Auml;ktionevent
   */
  void okButton_actionPerformed(ActionEvent e) {
    Integer fwPort;
    try {
      fwPort = new Integer(portField.getText());
    }
    catch(NumberFormatException exc) {
      exc.printStackTrace();
      fwPort = new Integer(8080);
      //ToDo: LogAusgabe, dass eine falsche Portnummer angegeben wurde und
      //deshalb der Standard-Port 8080 verwendet wird.
    }

    theSetting.setFwHost(serverField.getText());
    theSetting.fwType = theSetting.fwTunnel;
      //^^ bisher auf diese Konstante gesetzt, da noch nicht im Dialog einstellbar
    theSetting.fwPort = fwPort.intValue();
    theSetting.setAnonymPassword(passwordField.getText());
    theSetting.setAnonymUser(userField.getText());
    if (binaryRadio.isSelected()){
      theSetting.transfermode = theSetting.tmBinary;
    }
    else {
      theSetting.transfermode = theSetting.tmASCII;
    }
    dispose();
  }
}