package EasyFile;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.event.*;

/**
 * Hauptdialogfenster der Anwendung EasyFile. Verantwortlich
 * f&uuml;r die Erstellung und Verwaltung der grafischen
 * Benutzeroberfl&auml;che zur Klasse {@link Mdl_Ftp Mdl_Ftp}. Die
 * Oberfl&auml;che enth&auml;lt die &uuml;blichen Dialogelemente
 * eines Ftp-Clients. <br> Die Erzeugung dieses Hauptdialogfensters
 * ist in der Klasse {@link EasyFileCtrl EasyFileCtrl} als
 * Beispiel implementiert.
 *
 * @author Matthias Ansorg, Henrik Warnecke
 * @version 0.3
 */

public class GUI_HauptDlg extends JFrame implements Mdl_FtpEventListener
{
  //Nice:
  //-- Mdl_Hotlist.hibernate() und Mdl_Hotlist.awake() implementieren
  //-- Auswahl des fwTyps in GUI_EinstgDlg einbauen
  //-- Knopf Lschen im Verbindungsdialog einbauen.
  //-- Leeren Eintrag aus der ComboBox im Verbindungsdialog ausbauen
  //-- Rendering der Mdl_Connection als Strings soll die Zeichen : und @ nur
  //   bei Bedarf einbauen.
  //-- Passwortfeld im Verbindungsdialog durch ein Textfeld ersetzen
  //-- Alphabetische Sortierung der Eintrge in den Verzeichnislisten einbauen
  //-- Behandlung der NullPointerException bei cdLocal, wenn vom Verzeichnis
  //   / auf die Ebene der Dateisystemwurzeln gewechselt werden soll.
  //-- Fehler: Aufbau einer identischen Verbindung ohne vorherige Trennung der
  //   vorigen Verbindung fhrt dazu, dass keine Verzeichniseintrge angezeigt
  //   werden.
  //-- Vor dem Erstellen einer neuen Verbindung muss automatisch die alte
  //   getrennt werden, sonst entsteht eine IPWorksException "You cannot change
  //   theRemoteHost at this time. A connection is in progress."
  //
  //ToDo:
  //-- Verwendung eines Proxy-Servers soll funktionieren. Dazu den Typ korrekt
  //   einstellen.
  //-- Verzeichnislistings funktionieren nur unzuverlssig und tw. unvollstndig,
  //   bei anonymem Login nie.
  //   Bei Beobachtung mit Debugger werden stets alle Ereignisse erzeugt und
  //   angezeigt, bei freiem Lauf des Programms nicht. Das trotz eines interrupt()
  //   vor jedem Verzeichnislisting und ohne dass sich die Listings berschneiden.
  //   Vermutung: es liegt an der Serialisierung beim Debugging, also an falscher
  //   Thread-Implementierung.
  //   Denn was passiert, wenn GUI_HauptDlg.DirEntry(e) in Mdl_Ftp.fireDirEntryEvent(...)
  //   so schnell hintereinander aufgerufen wird, dass ein voriger Aufruf noch nicht
  //   beendet ist?
  //   Wer sagt denn, dass das StartDirEvent zuerst ankommt? (Race-Condition)

  Mdl_Ftp theFtp;
  Mdl_Hotlist theHotlist;
  Mdl_Connection theConnection;
  Mdl_Log theLog;
  Mdl_Setting theSetting;

  JMenuBar Menu = new JMenuBar();
  JMenu Verbindung = new JMenu();
  JMenuItem VerbdgHerstellen = new JMenuItem();
  JMenuItem VerbdgTrennen = new JMenuItem();
  JMenuItem Einstellungen = new JMenuItem();
  JMenuItem FehlerMldg = new JMenuItem();
  JMenuItem Beenden = new JMenuItem();
  JMenu Hotlist = new JMenu();
  JMenuItem aktVerbindung = new JMenuItem();
  JMenuItem verwalten = new JMenuItem();
  JMenu Hilfe = new JMenu();
  JMenuItem Hilfethemen = new JMenuItem();
  JMenuItem Info = new JMenuItem();

  JPanel rootPanel;

  BorderLayout borderLayout1 = new BorderLayout();
  FlowLayout flowLayout2 = new FlowLayout();
  FlowLayout flowLayout4 = new FlowLayout();

  JSplitPane Split_Vert = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
  Box Box_ftp   = new Box(BoxLayout.Y_AXIS);
  Box Box_lokal = new Box(BoxLayout.Y_AXIS);
  GridBagLayout gridBagLayout2 = new GridBagLayout();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  JPanel Panel_lokal = new JPanel();
  JPanel Panel_ftp = new JPanel();
  JLabel Label_lokal = new JLabel();
  JLabel Label_ftp = new JLabel();

  PlainDocument Doc_Dir_lokal = new PlainDocument();
  JTextField Dir_lokal = new JTextField(Doc_Dir_lokal,"",25);
  PlainDocument Doc_Dir_ftp = new PlainDocument();
  JTextField Dir_ftp = new JTextField(Doc_Dir_ftp,"",25);

  DefaultBoundedRangeModel ProgressBarMdl =
    new DefaultBoundedRangeModel(0, 0, 0, 100);
  JProgressBar ProgressBar = new JProgressBar(ProgressBarMdl);

  FlowLayout flowLayout5 = new FlowLayout();
  JButton Button_Stop = new JButton();
  JButton Button_Start = new JButton();
  JPanel Panel_Buttons = new JPanel();
  JButton Button_Dir_lokal = new JButton();
  JButton Button_Dir_ftp = new JButton();

  //Erzeugen der beiden JLists mit Scrollbars zum Anzeigen des aktuellen
  //Verzeichnisses auf dem lokalen System bzw. dem FTP-Server
  DefaultListModel ListFtpModel = new DefaultListModel();
  JList ListFtp = new JList(ListFtpModel);
  JScrollPane FtpScrollPane = new JScrollPane(ListFtp);

  DefaultListModel ListLokalModel = new DefaultListModel();
  JList ListLokal = new JList(ListLokalModel);
  JScrollPane LokalScrollPane = new JScrollPane(ListLokal);


/**
 * Konstruktion des Hauptdialogs unter Verwendung aller f&uuml;r
 * den Betrieb der Anwendung n&ouml;tigen Instanzen der Modellklassen. Auch
 * in GUI_HauptDlg nicht ben&ouml;tigte Objekte der Modellklassen werden
 * dem Konstruktor als Parameter &uuml;bergeben, sofern es sich um solche
 * Modellklassen handelt, die anwendungsweit nur in einer Instanz vorkommen.
 * Diese Instanz wird von GUI_HauptDlg beim Aufruf weiterer Dialoge
 * &uuml;bergeben.
 * @param theFtp Anwendungsweit vewendete, einzige Instanz der Klasse
 * {@link Mdl_Ftp Mdl_Ftp}.
 * @param theHotlist Anwendungsweit vewendete, einzige Instanz der Klasse
 * {@link Mdl_Hotlist Mdl_Hotlist}.
 * @param theLog Anwendungsweit vewendete, einzige Instanz der Klasse
 * {@link Mdl_Log Mdl_Log}.
 * @param theSetting Anwendungsweit vewendete, einzige Instanz der Klasse
 * {@link Mdl_Setting Mdl_Setting}.
 */

  public GUI_HauptDlg(Mdl_Ftp theFtp, Mdl_Hotlist theHotlist,
                      Mdl_Log theLog, Mdl_Setting theSetting)
  {
    this.theFtp = theFtp;
    this.theHotlist = theHotlist;
    this.theConnection = theConnection;
    this.theLog = theLog;
    this.theSetting = theSetting;

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
      theFtp.addMdl_FtpEventListener(this);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    //das aktuelle lokale Verzeichnis anzeigen
    theFtp.lsLocal();
  }

  //Initialisierung der Komponenten
  private void jbInit() throws Exception
  {
    //fr spter: setIconImage(Toolkit.getDefaultToolkit().createImage(GUI.class.getResource("[Ihr Symbol]")));

    this.setSize(new Dimension(800, 600));
    this.setTitle("EasyFile - ");

    //Meneintrge vom ersten bis zum letzten initialisieren
    Verbindung.setText("Verbindung");
    VerbdgHerstellen.setText("Verbindung herstellen");
    VerbdgHerstellen.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        VerbdgHerstellen_actionPerformed(e);
      }
    });
    VerbdgTrennen.setText("Verbindung trennen");
    VerbdgTrennen.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        VerbdgTrennen_actionPerformed(e);
      }
    });
    Einstellungen.setText("Einstellungen ...");
    Einstellungen.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Einstellungen_actionPerformed(e);
      }
    });
    FehlerMldg.setText("Fehlermeldungen anzeigen");
    FehlerMldg.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        FehlerMldg_actionPerformed(e);
      }
    });
    Beenden.setText("Beenden");
    Beenden.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
           Beenden_actionPerformed(e);
      }
    });
    Hotlist.setText("Hotlist");
    aktVerbindung.setText("aktuelle Verbindung zur Hotlist hinzufgen");
    aktVerbindung.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        aktVerbindung_actionPerformed(e);
      }
    });
    verwalten.setText("Hotlist verwalten");
    verwalten.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        verwalten_actionPerformed(e);
      }
    });
    Hilfe.setText("Hilfe");
    Hilfethemen.setText("Hilfethemen");
    Hilfethemen.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Hilfethemen_actionPerformed(e);
      }
    });
    Info.setText("Info");
    Info.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Info_actionPerformed(e);
      }
    });

    //Men bottom-up aufbauen
    Verbindung.add(VerbdgHerstellen);
    Verbindung.add(VerbdgTrennen);
    Verbindung.add(Einstellungen);
    Verbindung.add(FehlerMldg);
    Verbindung.addSeparator();
    Verbindung.add(Beenden);
    Hotlist.add(aktVerbindung);
    Hotlist.add(verwalten);
    Hilfe.add(Hilfethemen);
    Hilfe.addSeparator();
    Hilfe.add(Info);
    Menu.add(Verbindung);
    Menu.add(Hotlist);
    Menu.add(Hilfe);
    this.setJMenuBar(Menu);


    //rootPanel initialisieren
    rootPanel = (JPanel)this.getContentPane();
//    rootPanel.setPreferredSize(new Dimension(640, 480)); //noetig ?
    rootPanel.setLayout(new GridBagLayout());

/*        GridBagConstraints(int gridx, int gridy,
                             int gridwidth, int gridheight,
                             double weightx, double weighty,
                             int anchor, int fill,
                             Insets insets, int ipadx, int ipady)
*/
    //Aufbau der Enthaltenseinshierachie Ebene 1
    rootPanel.add(Split_Vert,
      new GridBagConstraints(0, 0,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                             new Insets(0, 0, 0, 0), 0, 0));
    rootPanel.add(Panel_Buttons,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             GridBagConstraints.REMAINDER, GridBagConstraints.REMAINDER,
                             1.0, 0.0,
                             GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                             new Insets(0, 0, 0, 0), 0, 0));

    //Aufbau der Enthaltenseinshierachie Ebene 2
    //Split_Vert.setDividerLocation(this.getWidth() / 2);
    Split_Vert.setResizeWeight(0.5);
    Split_Vert.add(Box_ftp, JSplitPane.RIGHT);
    Split_Vert.add(Box_lokal, JSplitPane.LEFT);

    Panel_Buttons.setLayout(flowLayout5);
    Panel_Buttons.add(Button_Dir_lokal, null);
    Panel_Buttons.add(Button_Start, null);
    Panel_Buttons.add(ProgressBar, null);
    Panel_Buttons.add(Button_Stop, null);
    Panel_Buttons.add(Button_Dir_ftp, null);

    //Aufbau der Enthaltenseinshierachie Ebene 3
    Box_ftp.add(Panel_ftp);
    Box_ftp.add(FtpScrollPane);

    Box_lokal.add(Panel_lokal);
    Box_lokal.add(LokalScrollPane);

    Button_Dir_lokal.setText("neues Verzeichnis");
    Button_Dir_lokal.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Button_Dir_actionPerformed(e);
      }
    });
    Button_Dir_ftp.setText("neues Verzeichnis");
    Button_Dir_ftp.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Button_Dir_actionPerformed(e);
      }
    });
    Button_Stop.setText("Abbrechen");
    Button_Stop.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Button_Stop_actionPerformed(e);
      }
    });
    Button_Start.setText(">>");
    Button_Start.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Button_Start_actionPerformed(e);
      }
    });
    ProgressBar.setStringPainted(true);

    //Aufbau der Enthaltenseinshierachie Ebene 4
    Panel_ftp.setLayout(gridBagLayout1);
    Panel_ftp.setPreferredSize(new Dimension(400, 30));
/*        GridBagConstraints(int gridx, int gridy,
                             int gridwidth, int gridheight,
                             double weightx, double weighty,
                             int anchor, int fill,
                             Insets insets(top,left,bottom,right), int ipadx, int ipady)
*/
    Panel_ftp.add(Label_ftp,
      new GridBagConstraints(0, 0,
                             1, 1,
                             0.0, 1.0,
                             GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    Panel_ftp.add(Dir_ftp,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 0,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    FtpScrollPane.setPreferredSize(new Dimension(400, 1400));
    ListFtp.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    ListFtp.setSelectionBackground(Color.orange);
    ListFtp.setCellRenderer(new DirCellRenderer());
    ListFtp.addMouseListener(new DirListMouseListener(ListFtp,Dir_ftp,theFtp));
    ListFtp.addListSelectionListener(
      new EnsureOnlySelectionListener(ListLokal,EnsureOnlySelectionListener.isLokal,Button_Start)
    );

    Panel_lokal.setLayout(gridBagLayout2);
    Panel_lokal.setPreferredSize(new Dimension(400, 30));
    Panel_lokal.add(Label_lokal,
      new GridBagConstraints(0, 0,
                             1, 1,
                             0.0, 1.0,
                             GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    Panel_lokal.add(Dir_lokal,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 0,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    LokalScrollPane.setPreferredSize(new Dimension(400, 1400));
    ListLokal.setCellRenderer(new DirCellRenderer());
    ListLokal.addMouseListener(new DirListMouseListener(ListLokal,Dir_lokal,theFtp));
    ListLokal.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    ListLokal.addListSelectionListener(
      new EnsureOnlySelectionListener(ListFtp,EnsureOnlySelectionListener.isFtp,Button_Start)
    );

    //Aufbau der Enthaltenseinshierachie Ebene 5
      //die bevorzugte Hhe wird vom hier verwandten vertikalen BoxLayout benutzt:
    Label_ftp.setPreferredSize(new Dimension(70, 30));
    Label_ftp.setText("ftp://");
    Dir_ftp.setText("");
    Dir_ftp.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Dir_ftp_actionPerformed(e);
      }
    });
      //die bevorzugte Hhe wird vom hier verwandten vertikalen BoxLayout benutzt:
    Label_lokal.setPreferredSize(new Dimension(70, 30));
    Label_lokal.setText("lokal");
    Dir_lokal.setText("/");
    Dir_lokal.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Dir_lokal_actionPerformed(e);
      }
    });
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Verbindung | Verbindung
 * herstellen&laquo;; stellt eine Ftp-Verbindung her durch Aufruf des Dialogs
 * {@link GUI_VerbdgDlg GUI_VerbdgDlg}. In diesem Dialog werden die
 * Verbindungsdaten eingegeben, die Verbindung erstellt und das erste
 * Directory-Listing vom Ftp-Server angefordert. &Uuml;ber ein eventuelles
 * Misslingen des Versuchs, eine Verbindung mit den angegebenen Einstellungen
 * herzustellen, wird der Benutzer im Log-Dialog informiert.
 */

  void VerbdgHerstellen_actionPerformed(ActionEvent e) {
    GUI_VerbdgDlg dlg = new GUI_VerbdgDlg(this,theFtp,theHotlist);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }

  /**
   * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Verbindung | Verbindung
   * trennen&laquo;; trennt eine bestehende FTP-Verbindung. Wenn keine
   * FTP-Verbindung besteht, passiert nichts, entsprechend der Spezifikation von
   * {@link Mdl_Ftp#removeConnection() Mdl_Ftp#removeConnection()}.
   */

  void VerbdgTrennen_actionPerformed(ActionEvent e) {
    theFtp.removeConnection();
      //setzt gleichzeitig die Verbindungsdaten auf leere Strings
    ListFtpModel.clear();
    Label_ftp.setText("ftp://");
    Dir_ftp.setText("");
  }

  /**
   * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Verbindung |
   * Einstellungen ...&laquo;; ruft den Einstellungsdialog {@link GUI_EinstgDlg
   * GUI_EinstgDlg} auf und zeigt ihn an. Darin kann der Benutzer anwendungsweite
   * Einstellungen vornehmen, z.B. Proxy-Server und Proxy-Port.
   */
  void Einstellungen_actionPerformed(ActionEvent e) {
    GUI_EinstgDlg dlg = new GUI_EinstgDlg(this, theSetting);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Verbindung |
 * Fehlermeldungen ...&laquo;; ruft den Log-Dialog {@link GUI_LogDlg GUI_LogDlg} auf
 * und zeigt ihn an. Darin werden alle Log-Meldungen seit dem letzten Start der
 * Anwendung EasyFile aufgelistet. Wenn der Anwender den Men&uuml;eintrag
 * &raquo;Verbindung | Fehlermeldungen ...&laquo; mehrfach aufruft, wird jedesmal ein
 * neuer Log-Dialog ge&ouml;ffnet, es k&ouml;nnen also mehrere Log-Dialoge
 * gleichzeitig ge&ouml;ffnet sein.
 */

  void FehlerMldg_actionPerformed(ActionEvent e) {
    GUI_LogDlg dlg = new GUI_LogDlg(this,theLog);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.show();
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Verbindung |
 * Beenden&laquo;; der Prozess der Anwendung EasyFile wird beendet.
 */

  void Beenden_actionPerformed(ActionEvent e)
  {
    System.exit(0);
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Hotlist | aktuelle
 * Verbindung zur Hotlist hinzuf&uuml;gen&laquo;
 */
  void aktVerbindung_actionPerformed(ActionEvent e) {
    theHotlist.addEntry(theFtp.connection);
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Hotlist | Hotlist
 * verwalten&laquo;; ruft den Verbindungsdialog {@link GUI_VerbdgDlg
 * GUI_VerbdgDlg} auf und zeigt ihn an. Hier kann der Benutzer Eintr&auml;ge
 * der Hotlist l&ouml;schen und neue hinzuf&uuml;gen, au&szlig;erdem
 * Verbindungen zu Ftp-Servern erstellen.
 */

  void verwalten_actionPerformed(ActionEvent e) {
    GUI_VerbdgDlg dlg = new GUI_VerbdgDlg(this, theFtp, theHotlist);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Hilfe |
 * Hilfethemen&laquo;; ruft den Hilfe-Dialog {@link GUI_HilfeDlg GUI_HilfeDlg}
 * auf und zeigt ihn an.
 */

  void Hilfethemen_actionPerformed(ActionEvent e) {
    GUI_HilfeDlg dlg = new GUI_HilfeDlg(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.show();
  }

/**
 * Aufgerufen bei Auswahl des Men&uuml;eintrags &raquo;Hilfe | Info&laquo;;
 * ruft den Info-Dialog {@link GUI_InfoDlg GUI_InfoDlg} auf und zeigt ihn an.
 */

  void Info_actionPerformed(ActionEvent e) {
    GUI_InfoDlg dlg = new GUI_InfoDlg(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }

  //Es folgen Ereignisbehandlungsroutinen fr Ereignisse, die nicht
  //durch den Aufruf von Meneintrgen erzeugt wurden.

/**
 * Aufgerufen beim Bet&auml;tigen eines der beiden Kn&ouml;pfe &raquo;neues
 * Verzeichnis&laquo;; ruft den Dialog zum Erstellen neuer Verzeichnisse
 * {@link GUI_MkdirDlg GUI_MkdirDlg} auf und zeigt ihn an. Da mit dem Dialog
 * entweder ein Verzeichnis auf dem lokalen System oder auf dem Server
 * erzeugt werden kann, k&ouml;nnen zwei (oder mehr) Dialoge gleichzeitig
 * ge&ouml;ffnet sein.
 * @param e Das beim Bet&auml;tigen eines Knopfes erzeugte Ereignis. An seiner
 * Herkunft kann erkannt werden, ob ein Verzeichnis auf dem Ftp-Server oder
 * dem lokalen System erzeugt werden soll.
 */

  void Button_Dir_actionPerformed(ActionEvent e) {
    GUI_MkdirDlg dlg;

    if (e.getSource() == Button_Dir_lokal)
    {
      dlg = new GUI_MkdirDlg(this, theFtp, GUI_MkdirDlg.dirOnLocal);
    }
    else
    {
      dlg = new GUI_MkdirDlg(this, theFtp, GUI_MkdirDlg.dirOnServer);
    }
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }

  /**
   * Aufgerufen beim Bet&auml;tigen des Knopfes &raquo;>>&laquo;; startet eine
   * Datei&uuml;bertragung, sofern eine Datei in einer Verzeichnisliste (entweder
   * die lokale oder die f&uuml;r den FTP-Server) markiert ist.
   */
  void Button_Start_actionPerformed(ActionEvent e) {
    DirEntryEvent dirListEntryFTP = (DirEntryEvent)ListFtp.getSelectedValue();
    DirEntryEvent dirListEntryLokal = (DirEntryEvent)ListLokal.getSelectedValue();
    //Vorsicht: vielleicht mssen Threads verwendet werden, damit die
    //Fortschrittsanzeige funktioniert. Vgl.
    // http://java.sun.com/docs/books/tutorial/uiswing/ovrview/event.html

    //entweder ist ein Wert in der Liste ListFTP selektiert (Abfrage true)
    //XOR in der Liste ListLokal (Abfrage false), aber nie in beiden:
    if (dirListEntryFTP!=null) {
      //selektierte Datei downloaden; es werden die aktuell angezeigten
      //Verzeichnisse der Listen verwendet, sie mssen bereits in Mdl_Ftp
      //entsprechend gesetzt sein
      if (! dirListEntryFTP.isDir)
        theFtp.download(dirListEntryFTP.getDirEntry());
        theFtp.lsLocal();
        clearListFtpModel();
        theFtp.lsServer();
          //ein Hack. Weil bei download(...) ein StartDirEvent ausgelst wird,
          //das das Lschen der Ftp-Liste bewirkt.
    }
    else if (dirListEntryLokal!=null) {
      //selektierte Datei uploaden, analog zum downloaden
      if (! dirListEntryLokal.isDir)
        theFtp.upload(dirListEntryLokal.getDirEntry());
        clearListFtpModel();
        theFtp.lsServer();
        theFtp.lsLocal();
          //ein Hack. Weil bei upload(...) ein StartDirEvent ausgelst wird,
          //das das Lschen der Local-Liste bewirkt.
    }
  }

/**
 * Aufgerufen beim Bet&auml;tigen des Knopfes &raquo;>>&laquo;; bricht eine
 * laufende Datei&uuml;bertragung ab, ohne jedoch die Verbindung zum FTP-Server
 * ganz zu schlie&szlig;en. Wenn keine Datei&uuml;bertragung stattfindet,
 * geschieht nichts.
 */

  void Button_Stop_actionPerformed(ActionEvent e) {
    theFtp.cancelTransfer();
  }

/**
 * Vom ActionListener des Textfeldes Dir_lokal aufgerufen, wenn eine Eingabe mit
 * der Taste &raquo;Return&laquo; best&auml;tigt wurde; es wird in das
 * eingegebene Unterverzeichnis des aktuellen Verzeichnisses gewechselt. Es
 * wird immer ein Wechsel in das eingegebene Verzeichnis angezeigt, auch wenn
 * dies aufgrund von Zugriffsrechten eigentlich unm&ouml;glich ist. In einem
 * solchen Fall ist das Verzeichnislisting leer bis auf den Eintrag
 * &raquo;..&laquo;, mit dem der Benutzer wieder ins &uuml;bergeordnete
 * Verzeichnis wechseln kann.
 */

  void Dir_lokal_actionPerformed(ActionEvent e) {
    String dirEntryLokal = Dir_lokal.getText();
    theFtp.cdLocal(new File(dirEntryLokal).getName());
    theFtp.lsLocal();
  }

/**
 * Vom ActionListener des Textfeldes Dir_ftp aufgerufen, wenn eine Eingabe mit
 * der Taste &raquo;Return&laquo; best&auml;tigt wurde; es wird in das
 * eingegebene Unterverzeichnis des aktuellen Verzeichnisses gewechselt. Es
 * wird immer ein Wechsel in das eingegebene Verzeichnis angezeigt, auch wenn
 * dies aufgrund von Zugriffsrechten eigentlich unm&ouml;glich ist. In einem
 * solchen Fall ist das Verzeichnislisting leer bis auf den Eintrag
 * &raquo;..&laquo;, mit dem der Benutzer wieder ins &uuml;bergeordnete
 * Verzeichnis wechseln kann.
 */

  void Dir_ftp_actionPerformed(ActionEvent e) {
    String dirEntryFtp = Dir_ftp.getText();
    theFtp.interrupt();
      //ggf. stattfindendes Verzeichnislisting abbrechen, sonst entsteht
      //ein Durcheinander von Ereignissen.
    theFtp.cdServer(dirEntryFtp);
    clearListFtpModel();
    theFtp.lsServer();
  }

  //Es folgen Methoden, die das Interface Mdl_FtpEventListener implementieren

/**
 * Reagiert auf ein {@link progressEvent progressEvent} durch geeignete
 * &Auml;nderungen an der {@link JProgressBar ProgressBar}.
 */

  public void progress(progressEvent e) {
    ProgressBar.setModel(new DefaultBoundedRangeModel(e.progress,0,0,100));
    ProgressBar.setString(Integer.toString(e.progress,10)+"%");
  }

  public void StartDir(StartDirEvent e) {
    if (e.place==e.isFtp) {
      Label_ftp.setText("ftp://" + theFtp.connection.getFtphost());
//    ListFtpModel.clear();
        //blo nicht hier, denn aufgrund Multithreading ist nicht garantiert,
        //dass dieses Event vor allen DirEntryEvents ankommt (Race-Condition).
        //Das kann bedeuten, dass mit dieser Anweisung die Liste irgendwann
        //whrend des Listings einmal gelscht wird. Stattdessen wird diese
        //Anweisung vor jeder Anforderung eines Listings ausgefhrt.
    }
    else {
      //Erzeuge Verzeichniseintrag, der das bergeordnete Verzeichnis meint
      //bei Ftp-Verzeichnislistings bereits automatisch vorhanden.
      DirEntryEvent parentDir = new DirEntryEvent(this);
      parentDir.setDirEntry("..");
      parentDir.isDir=true;
      parentDir.place=e.place;

      ListLokalModel.clear();
      ListLokalModel.add(0,parentDir); //Eintrag, der das bergeordnete Verzeichnis meint
    }
  }

  public void DirEntry(DirEntryEvent e) {
    if (e.place==e.isFtp) {
      ListFtpModel.addElement(e);
    }
    else {
      ListLokalModel.addElement(e);
    }
  }

  public void StopDir(StopDirEvent e) {
    //keine Aktion notwendig, Funktion muss aber angeboten werden, da sie
    //zum Interface Mdl_FtpEventListener gehrt.
  }

  /**
   * Muss vor jeder Anforderung eines Ftp-Verzeichnislistings (mit
   * {@link Mdl_Ftp#lsServer() Mdl_Ftp.lsServer()} aufgerufen werden. Diese
   * Aktion kann nicht mit einem StartDirEvent
   * ({@link #StartDir(StartDirEvent e) StartDir(StartDirEvent e)} verbunden
   * werden, weil aufgrund des Multithreading (&raquo;EventDispatcherThread&laquo;)
   * nicht garantiert ist, dass dieses Ereignis zuerst ankommt! Diese Methode
   * ist ein Hack, um bisher eine saubere Thread-Synchronisierung zu vermeiden.
   * Das muss in spteren Versionen gendert werden.
   */
  public void clearListFtpModel() {
    ListFtpModel.clear();
  }

  //berschrieben, so da eine Beendigung beim Schlieen des Fensters mglich ist.
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      Beenden_actionPerformed(null);
    }
  }
}


class DirCellRenderer extends JLabel implements ListCellRenderer {
  final static ImageIcon fileIcon = new ImageIcon("file.gif");
  final static ImageIcon dirIcon  = new ImageIcon("dir.gif");

  public DirCellRenderer() {
    //nicht vergessen, sonst werden vorhandene Selektionen nicht gezeichnet:
    setOpaque(true);
  }

  public Component getListCellRendererComponent(
    JList list,
    Object dirListEntry,     // ein Eintrag der Verzeichnisliste
    int index,               // Indexnummer der Zelle dieses Eintrags in der Verzeichnisliste
    boolean isSelected,      // ist die Zelle selektiert?
    boolean cellHasFocus)    // haben Liste und Zelle gleichzeitig den Fokus?
  {
    DirEntryEvent extractedEvent = (DirEntryEvent)dirListEntry;
    String s = extractedEvent.getDirEntry();
    setText(s);
    //Am Icon von JLabel soll man erkennen, ob es sich um eine Datei oder um
    //ein Verzeichnis handelt:
    setIcon((extractedEvent.isDir) ? dirIcon : fileIcon);
    if (isSelected) {
      setBackground(list.getSelectionBackground());
      setForeground(list.getSelectionForeground());
    }
    else {
      setBackground(list.getBackground());
      setForeground(list.getForeground());
    }
    setEnabled(list.isEnabled());
    setFont(list.getFont());
    return this;
  }
}
/**
 * Implementierung eines MouseListener zur Behandlung von Doppelklicks in den
 * JLists zur Darstellung des aktuellen und des Ftp-Verzeichnisses. Wenn es
 * sich beim angeklickten Eintrag um ein Verzeichnis handelt, soll in dieses
 * Verzeichnis gewechselt werden. Die Funktionen cdServer(String) und cdLocal(String)
 * zeigen auch dann einen gelungenen Wechsel des aktuellen Verzeichnisses an,
 * wenn dies aufgrund der Zugriffsrechte nihct m&ouml;glich ist. In diesem Fall
 * liefern lsServer() und lsLocal() ein leeres Verzeichnislisting, so dass
 * der Benutzer wieder ins &uuml;bergeordnete Verzeichnis wechselt.
 */

class DirListMouseListener extends MouseAdapter {
  JList list;
  Mdl_Ftp theFtp;
  JTextField dirField;

  public DirListMouseListener(JList list, JTextField dirField, Mdl_Ftp theFtp) {
    super();
    this.list=list;
    this.theFtp=theFtp;
    this.dirField=dirField;
  }
  public void mouseClicked(MouseEvent e) {
    int index = list.locationToIndex(e.getPoint());
    final int clickCount = e.getClickCount();
    //Bei Doppelklick auf ein Verzeichnis in das Verzeichnis
    //wechseln. Selektionen (d.h. Einzelklicks) werden durch den
    //ListSelectionListener EnsureOnlySelectionListener verwaltet.
    if (clickCount == 2) {
      //ToDo: Bei Doppelklick soll nichts selektiert werden!
      //es kann also nicht selectedValue ausgewertet werden, sondern index
      DirEntryEvent dirListEntry=(DirEntryEvent)list.getSelectedValue();
      if (dirListEntry.isDir) {
        if (dirListEntry.place==dirListEntry.isFtp) {
          theFtp.cdServer(dirListEntry.getDirEntry());
          ((DefaultListModel)list.getModel()).clear();
            //ein Hack. quivalent zu GUI_HauptDlg.clearListFtpModel, jedoch ist
            //dieser Klasse bisher die anwendungsweite Instanz von GUI_HauptDlg
            //nicht bekannt.
          theFtp.lsServer();
        }
        else {
          theFtp.cdLocal(dirListEntry.getDirEntry());
          theFtp.lsLocal();
        }
        File actualDir;
        String parentDirString = dirField.getText();
        String childDirString = dirListEntry.getDirEntry();
        if (childDirString.equals(".."))
          //actualDir wird null, wenn kein bergeordnetes Verzeichnis existiert:
          actualDir = new File(parentDirString).getParentFile();
        else
          actualDir = new File(parentDirString,childDirString);
        if (actualDir != null) {
          dirField.setText(actualDir.toString());
        }
      }
    }
  }
}

/**
 * Verantwortlich f&uuml;r das L&ouml;schen aller Selektionen in einer
 * &uuml;bergebenen JList, wenn mindestens ein Element des von diesem
 * ListSelectionListener beobachteten Objektes selektiert ist. Damit ist es
 * m&ouml;glich, zwei JList so zu verwenden, dass nur Eintr&auml;ge in einer
 * JList selektiert sein k&ouml;nnen, indem man n&auml;mlich diesen
 * EnsureOnlySelectionListener als ListSelectionListener in beiden JLists
 * verwendet.
 */

class EnsureOnlySelectionListener implements ListSelectionListener {
  public static final int isFtp = 0;
  public static final int isLokal = 1;

  JList oppositeList;
  int oppositePos;
  JButton arrowButton;

  /**
   * Erstelle einen neuen EnsureOnlySelectionListener
   *
   * @param oppositeList diejenige der beiden Verzeichnislisten, von der diese
   * Instanz des EnsureOnlySelectionListeners nicht als ListSelectionListener
   * registriert wurde.
   * @param oppositePos gibt an, ob die oppositeList das lokale oder das
   * Ftp-Verzeichnis darstellt. Der Wert ist also entweder {@link isFtp isFtp}
   * oder {@link isLokal isLokal}.
   * @param arrowButton die Schaltfl&auml;che, mit der der Dateitansfer
   * gestartet werden kann. Die Verwaltung ihrer Beschriftung ist ebenfalls
   * Aufgabe dieser Klasse.
   */
  public EnsureOnlySelectionListener (JList oppositeList, int oppositePos, JButton arrowButton) {
    this.oppositeList = oppositeList;
    this.oppositePos  = oppositePos;
    this.arrowButton  = arrowButton;
  }

  /**
   * Wenn mindestens ein Element selektiert ist, darf in der anderen Liste
   * kein Element selektiert sein, d.h. deren Selektion wird aufgehoben.
   * Auerdem wird die Beschriftung des JButton arrowButton so gendert,
   * dass sie stets zu der Liste mit selektierten Objekten zeigt; wenn keine
   * der beiden Listen selektierte Objekte hat, ist die Beschriftung des
   * Buttons unbestimmt.
   */
  public void valueChanged(ListSelectionEvent e) {
    if (! ((JList)e.getSource()).isSelectionEmpty()) {
      oppositeList.clearSelection();
      //Beschriftung des Buttons hin zur oppositeList ndern
      if (oppositePos==isFtp)
        arrowButton.setText(">>");
      else
        arrowButton.setText("<<");
    }
  }
}