package EasyFile;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Stellt den Dialog zum Erzeugen neuer Verzeichnisse des
 * Ftp-Clients EasyFile dar. <code>GUI_MkdirDlg</code> benutzt
 * {@link Mdl_Ftp#mkdirServer(String) Mdl_Ftp.mkdirServer(String directory)}
 * und
 * {@link Mdl_Ftp#mkdirLocal(String) Mdl_Ftp.mkdirLocal(String directory)}
 * um die Verzeichnisse anzulegen.
 * @author Ralf Mei&szlig;ner, Matthias Ansorg
 * @version 0.3
 */

public class GUI_MkdirDlg extends JDialog {

  /**
   * Konstante, die als Parameter des Konstruktors von
   * <code>GUI_MkdirDlg</code> verwendet wird und angibt,
   * dass ein Verzeichnis auf dem Ftp-Server erzeugt werden soll.
   */
  public static final int dirOnServer=0;

  /**
   * Konstante, die als Parameter des Konstruktors von
   * <code>GUI_MkdirDlg</code> verwendet wird und angibt,
   * dass ein Verzeichnis auf dem lokalen System erzeugt werden soll.
   */
  public static final int dirOnLocal=1;

  int dirOn;
  Mdl_Ftp theFtp;
  JPanel rootPanel = new JPanel();
  JButton cancelButton = new JButton();
  JButton makeButton = new JButton();
  JLabel dirLabel = new JLabel();
  JTextField dirField = new JTextField();
  GridBagLayout gridBagLayout = new GridBagLayout();

  /**
   * Erzeugt den Dialog zum Erzeugen neuer Verzeichnisse des Ftp-Clients EasyFile.
   *
   * @param parent  Das Fenster, von dem der Benutzer den Dialog
   * <code>GUI_MkdirDlg</code> aufgerufen hat.
   * @param dirOn   Gibt an, ob ein Verzeichnis auf dem Ftp-Server (Wert
   * {@link #dirOnServer dirOnServer}==0) oder auf dem lokalen System (Wert
   * {@link #dirOnLocal dirOnLocal}==1) erzeugt werden soll.
   */
  public GUI_MkdirDlg(Frame parent,Mdl_Ftp theFtp, int dirOn) {
    super(parent);
    this.theFtp = theFtp;
    this.dirOn = dirOn;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Initialisierung der Komponenten.
   */
  private void jbInit() throws Exception {
    this.setSize(new Dimension(400,100));
    this.setTitle("EasyFile - Verzeichnis erstellen");
    this.getContentPane().add(rootPanel, BorderLayout.CENTER);

    rootPanel.setLayout(gridBagLayout);
/*        GridBagConstraints(int gridx, int gridy,
                             int gridwidth, int gridheight,
                             double weightx, double weighty,
                             int anchor, int fill,
                             Insets insets(top,left,bottom,right), int ipadx, int ipady)
*/
    rootPanel.add(dirLabel,
      new GridBagConstraints(0, 0,
                             1, 1,
                             0.5, 1.0,
                             GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 0, 0), 0, 0));
    rootPanel.add(dirField,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 0,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 0, 0, 5), 0, 0));
    rootPanel.add(makeButton,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             0.5, 0.5,
                             GridBagConstraints.EAST, GridBagConstraints.NONE,
                             new Insets(0, 5, 5, 0), 0, 0));
    rootPanel.add(cancelButton,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 1,
                             1, 1,
                             1.0, 0.5,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 5), 0, 0));

    cancelButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancelButton_actionPerformed(e);
      }
    });
    makeButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        makeButton_actionPerformed(e);
      }
    });
    dirLabel.setText("Verzeichnisname");
    cancelButton.setText("Abbrechen");
    makeButton.setText("Verzeichnis erstellen");
  }

  /**
   * Beenden des Dialoges
   * @param e Ausl&ouml;sendes &Auml;ktionevent
   */
  void cancelButton_actionPerformed(ActionEvent e) {
    dispose();
  }

  /**
   * Verzeichnis wird abh&auml;ngig vom Parameter des Konstruktors auf dem Server oder dem lokalen Rechner angelegt
   * @param e Ausl&ouml;sendes &Auml;ktionevent
   */
  void makeButton_actionPerformed(ActionEvent e) {
    if (dirOn == dirOnServer) {
      theFtp.mkdirServer(dirField.getText());
      theFtp.lsServer();
    }
    else {
      if (dirOn == dirOnLocal) {
        theFtp.mkdirLocal(dirField.getText());
        theFtp.lsLocal();
      }
    }
    dispose();
  }
}