package EasyFile;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 * Dialog zum Einstellen der Verbindungsdaten f&uuml;r die Verbindung zu einem
 * FTP-Server. Er wird aufgerufen, wenn in EasyFile der Men&uuml;eintrag
 * &raquo;Verbindung | Verbindung herstellen&laquo; aufgerufen wird. Es
 * k&ouml;nnen Eintr&auml;ge der Hotlist {@link Mdl_Hotlist Mdl_Hotlist}
 * verwendet oder ein eigener Eintrag gemacht werden. Der Dialog ist auch
 * daf&uuml;r verantwortlich, durch Nutzung der Modell-Klassen tats&auml;chlich
 * die Verbindung aufzubauen und ein erstes Directory-Listing vom Ftp-Server
 * anzufordern.
 *
 * @author Matthias Ansorg, Ralf Mei&szlig;ner
 * @version 0.3
 */

public class GUI_VerbdgDlg extends JDialog {
  Mdl_Hotlist theHotlist;
  Mdl_Ftp theFtp;
  Mdl_Hotlistbox theHotlistboxMdl;

  GridBagLayout gridBagLayout = new GridBagLayout();

  JPanel rootPanel = new JPanel();

  JLabel titleLabel = new JLabel("Liste gespeicherter Ftp-Verbindungen:");

  /* Wird ein Eintrag der Hotlist ausgewhlt, so werden dessen Daten in die
   * Textfelder server_feld, verzeichnis_feld, user_feld und passwort_feld
   * eingetragen. Die Daten dieser Felder allein werden ausgewertet, wenn der
   * Knopf Verbinden bettigt wird. Der Benutzer kann also einen
   * Hotlist-Eintrag verwenden, ihn abndern und verwenden, oder vllig neue
   * Verbindungsdaten eingeben.
   * Erstelle eine ComboBox mit allen Eintrgen der Hotlist:
   */
  JComboBox hotlistbox = new JComboBox();

  JLabel thisConnectionLabel = new JLabel("Diese Ftp-Verbindung:");

  PlainDocument server_feldDoc      = new PlainDocument();
  PlainDocument verzeichnis_feldDoc = new PlainDocument();
  PlainDocument user_feldDoc        = new PlainDocument();
  PlainDocument passwort_feldDoc    = new PlainDocument();

  JLabel server_label = new JLabel();
  JTextField server_feld = new JTextField(server_feldDoc,"",25);
  JLabel verzeichnis_label = new JLabel();
  JLabel passwort_label = new JLabel();
  JLabel user_label = new JLabel();
  JTextField verzeichnis_feld = new JTextField(verzeichnis_feldDoc,"",25);
  JTextField user_feld = new JTextField(user_feldDoc,"",25);
  JPasswordField passwort_feld = new JPasswordField(passwort_feldDoc,"",25);
  JLabel logintitel_label = new JLabel();

  JButton verbinden = new JButton();
  JButton abbrechen = new JButton();
  JButton aendern   = new JButton();

/**
 * Erzeugen des Verbindungs-Dialogs.
 *
 * @param parent  Das Fenster, von dem der Benutzer den Hotlist-Dialog
 * aufgerufen hat.
 * @param theFtp die anwendungsweit benutzte Instanz von {@link Mdl_Ftp Mdl_Fto},
 * die ben&ouml;tigte Ftp-Funktionalit&auml;t bereitstellt.
 * @param theHotlist die anwendungsweit benutzte Instanz von
 * {@link Mdl_Hotlist Mdl_Hotlist}, die die Hotlist (Liste h&auml;ufig benutzter
 * Verbindungen) enth&auml;lt.
 */

  public GUI_VerbdgDlg(Frame parent, Mdl_Ftp theFtp, Mdl_Hotlist theHotlist) {
    super(parent);
    this.theHotlist = theHotlist;
    this.theFtp = theFtp;
    this.theHotlistboxMdl = new Mdl_Hotlistbox(this.theHotlist);
    //post: theHotlistboxMdl enthlt alle Eintrge von theHotlist

    hotlistbox.setModel(theHotlistboxMdl);
      //^^ enthlt theHotlistboxMdl.addListDataListener(hotlistbox);

//    //initialisiere hotlistbox mit den Eintrgen des verwendeten Modells
//    ComboBoxModel myModel = hotlistbox.getModel();
//    for (int i=0; i<myModel.getSize(); ++i)
//      hotlistbox.addItem(myModel.getElementAt(i));

//    System.err.println("Models current size: " + theHotlistboxMdl.getSize());
    hotlistbox.setSelectedIndex(0);
      //Eintrag mit ID 0 der Hotlist ist ein leerer Eintrag (siehe Mdl_Hotlist)
      //da die Indizes identisch sind (siehe Mdl_Hotlistbox), ist auch der
      //Eintrag mit ID 0 des Datenmodells theHotlistboxMdl ein leerer Eintrag.

    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {

    this.setSize(new Dimension(500,250));
    this.setTitle("EasyFile - Verbindungsverwaltung");
    this.getContentPane().add(rootPanel,BorderLayout.CENTER);

    rootPanel.setPreferredSize(new Dimension(500, 250));
    rootPanel.setLayout(gridBagLayout);
/*        GridBagConstraints(int gridx, int gridy,
                             int gridwidth, int gridheight,
                             double weightx, double weighty,
                             int anchor, int fill,
                             Insets insets(top,left,bottom,right), int ipadx, int ipady)
*/
    rootPanel.add(titleLabel,
      new GridBagConstraints(0, 0,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 0, 5), 0, 0));
    rootPanel.add(hotlistbox,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(thisConnectionLabel,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(server_label,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(server_feld,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 3,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(verzeichnis_label,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(verzeichnis_feld,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 4,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(logintitel_label,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 5), 0, 0));
    rootPanel.add(user_label,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(user_feld,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 6,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(passwort_label,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.NONE,
                             new Insets(0, 5, 0, 0), 0, 0));
    rootPanel.add(passwort_feld,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 7,
                             GridBagConstraints.REMAINDER, 1,
                             1.0, 1.0,
                             GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
    rootPanel.add(verbinden,
      new GridBagConstraints(0, GridBagConstraints.RELATIVE,
                             1, GridBagConstraints.REMAINDER,
                             1.0, 1.0,
                             GridBagConstraints.CENTER, GridBagConstraints.NONE,
                             new Insets(0, 5, 5, 0), 0, 0));
    rootPanel.add(abbrechen,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 8,
                             1,1,
                             1.0, 1.0,
                             GridBagConstraints.CENTER, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 0), 0, 0));
    rootPanel.add(aendern,
      new GridBagConstraints(GridBagConstraints.RELATIVE, 8,
                             GridBagConstraints.REMAINDER, GridBagConstraints.REMAINDER,
                             1.0, 1.0,
                             GridBagConstraints.CENTER, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 5), 0, 0));
    server_label.setText("FTP-Server");
    verzeichnis_label.setText("FTP-Verzeichnis");
    logintitel_label.setText("auer fr anonymes Login:");
    user_label.setText("Benutzername");
    passwort_label.setText("Passwort");
    verbinden.setText("Verbinden");
    verbinden.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        verbinden_actionPerformed(e);
      }
    });
    abbrechen.setText("Beenden");
    abbrechen.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        abbrechen_actionPerformed(e);
      }
    });
    aendern.setText("ndern");
    aendern.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e){
       aendern_actionPerformed(e);
      }
    });
    hotlistbox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        hotlistbox_actionPerformed(e);
      }
    });
  }

  /**
   * Wird &uuml;ber einen anonymen ActionListener aufgerufen, wenn der Benutzer
   * den Schalter &raquo;Verbinden&laquo; bet&auml;tigt hat. Es werden die
   * Verbindungsdaten aus den Eintr&auml;gen der Textfelder gesammelt und damit
   * eine Verbindung erstellt. Anschlie&szlig;end wird der Dialog geschlossen.
   */
  void verbinden_actionPerformed(ActionEvent e) {
    theFtp.connection.setFtphost(server_feld.getText());
    theFtp.connection.setFtpdir(verzeichnis_feld.getText());
    theFtp.connection.setPassword(new String(passwort_feld.getPassword()));
    theFtp.connection.setUsername(user_feld.getText());

    theFtp.createConnection();
      //lsServer wird bei gelungenem Verbindungsaufbau in createConnection ausgefuehrt
    cancel(); //Dialog schlieen
  }

  /**
   * Wird &uuml;ber einen anonymen ActionListener aufgerufen, wenn der Benutzer
   * den Schalter &raquo;Abbrechen&laquo; bet&auml;tigt hat. Der Dialog wird
   * einfach geschlossen.
   *
   * @param e das ausl&ouml;dende ActionEvent
   */
  void abbrechen_actionPerformed(ActionEvent e) {
    cancel();
  }

  /**
   * Wird &uuml;ber einen anonymen ActionListener aufgerufen, wenn der Benutzer
   * die Schaltfl&auml;che &raquo;&Auml;ndern&laquo; bet&auml;tigt. Der z. Zt.
   * selektierte Eintrag der ComboBox wird entsprechend den Daten in den
   * Textfeldern ge&auml;ndert, die Anzeige in der ComboBox &auml;ndert sich.
   *
   * @param e das ausl&ouml;dende ActionEvent
   */
  void aendern_actionPerformed(ActionEvent e){
    Mdl_Connection newSelectedItem =
      new Mdl_Connection(user_feld.getText(),
                         new String(passwort_feld.getPassword()),
                         server_feld.getText(),
                         verzeichnis_feld.getText()
                         );
    Mdl_Connection nowSelectedItem = (Mdl_Connection)theHotlistboxMdl.getSelectedItem();
    int            nowSelectedIndex = theHotlistboxMdl.getIndexOf(nowSelectedItem);
    theHotlistboxMdl.insertElementAt(newSelectedItem,nowSelectedIndex);
    //nowSelectedItem existiert nicht mehr, kann deshalb auch nicht selektiert sein.
    //deshalb neue Selektion festlegen:
    theHotlistboxMdl.setSelectedItem(newSelectedItem);
  }

  /**
   * Wird &uuml;ber einen anonymen ActionListener aufgerufen, wenn der Benutzer
   * einen Eintrag der JComboBox hotlistbox ausgew&auml;hlt hat. Entsprechend
   * den Daten dieses Eintrags wird der Inhalt der Textfelder im Dialog gesetzt.
   */
  void hotlistbox_actionPerformed(ActionEvent e) {
    JComboBox hotlistbox = (JComboBox)e.getSource();
    try {
      //hier tritt bisher eine ClassCastException auf:
      Mdl_Connection selectedConnection = (Mdl_Connection)theHotlistboxMdl.getSelectedItem();

      //Textfelder im Verbindungsdialog mit den Daten aus dem ausgewhlten
      //Eintrag der ComboBox fllen:
      server_feldDoc.remove(0,server_feldDoc.getLength());
      server_feldDoc.insertString(0,selectedConnection.getFtphost(),null);
      verzeichnis_feldDoc.remove(0,verzeichnis_feldDoc.getLength());
      verzeichnis_feldDoc.insertString(0,selectedConnection.getFtpdir(),null);
      user_feldDoc.remove(0,user_feldDoc.getLength());
      user_feldDoc.insertString(0,selectedConnection.getUsername(),null);
      passwort_feldDoc.remove(0,passwort_feldDoc.getLength());
      passwort_feldDoc.insertString(0,selectedConnection.getPassword(),null);
    }
    catch (Exception exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Den Verbindungsdialog schlie&szlig;en.
   */
  void cancel() {
    dispose();
  }
}