package EasyFile;

/**
 * Verantwortlich f&uuml;r die Verwaltung der Daten einer einzigen Verbindung
 * vom lokalen Rechner zu einem FTP-Server. Anwendungsweite
 * Verbindungseinstellungen werden von {@link Mdl_Setting Mdl_Setting}
 * verwaltet.
 * @author Matthias Ansorg
 * @version 0.3
 */
public class Mdl_Connection implements Cloneable{

  StringBuffer username = new StringBuffer();
  StringBuffer password = new StringBuffer();
  StringBuffer ftphost  = new StringBuffer();
  StringBuffer ftpdir   = new StringBuffer();

  /**
   * Erzeugt ein neues Mdl_Connection mit leeren Feldern f&uuml;r die
   * Verbindungsdaten.
   */
  public Mdl_Connection() {
  }

  /**
   * Erzeugt ein neues Mdl_Connection mit Initialisierung der Felder f&uuml;r
   * die Verbindungsdaten entsprechend den &uuml;bergebenen Parametern.
   * @param username Der Benutzername, unter dem sich der Benutzer auf dem
   * Ftp-Server einloggen will. Wird hier eine leere Zeichenkette
   * &uuml;bergeben, bedeutet das anonymes Login.
   * @param password Das Passwort zum Benutzernamen.
   * @param ftphost Der Ftp-Server, zu dem eine Verbindung aufgebaut werden
   * soll, in der Form &raquo;ftp://hostname.tld&laquo;.
   * @param ftpdir Verzeichnis, in das nach erfolgreichem Verbindungsaufbau
   * zum Ftp-Server gewechselt werden soll.
   */
  public Mdl_Connection(String username, String password,
                        String ftphost, String ftpdir)   {
    this.username.append(username);
    this.password.append(password);
    this.ftphost.append(ftphost);
    this.ftpdir.append(ftpdir);
  }

  /**
   * Den Benutzernamen, unter dem sich der Benutzer auf dem Ftp-Server
   * einloggen will, setzen.
   * @param username Der zu setzende Benutzername
   */
  public void setUsername(String username) {
    this.username.replace(0,this.username.length(),username);
  }

  /**
   * Liefert den Benutzernamen, der zu dieser mit Mdl_Connection beschriebenen
   * Ftp-Verbindung (d.h. zum Login mit dem Benuternamen) geh&ouml;rt.
   * @return der Benutzername dieser Ftp-Verbindung
   */
  public String getUsername() {
    return this.username.toString();
  }

  /**
   * Das zum Benutzernamen geh&ouml;rende Passwort zum Login auf dem Ftp-Server
   * setzen.
   * @param password Das zu setzende Passwort.
   */
  public void setPassword(String password) {
    this.password.replace(0,this.password.length(),password);
  }

  /**
   * Liefert das Passwort, das zu dieser mit Mdl_Connection beschriebenen
   * Ftp-Verbindung (d.h. zum Login mit dem Benuternamen) geh&ouml;rt.
   * @return das Passwort dieser Ftp-Verbindung
   */
  public String getPassword() {
    return this.password.toString();
  }

  /**
   * Den Ftp-Server, der zur mit Mdl_Connection beschriebenen Verbindung zu
   * einem Ftp-Server geh&ouml;rt.
   * @param ftphost Der Ftp-Server, anzugeben in der Form
   * &raquo;ftp://hostname.tld&laquo;.
   */
  public void setFtphost(String ftphost) {
    this.ftphost.replace(0,this.ftphost.length(),ftphost);
  }

  /**
   * Liefert den Namen des Ftp-Servers, der zu dieser mit Mdl_Connection
   * beschriebenen Ftp-Verbindung geh&ouml;rt.
   * @return der Name des Ftp-Servers dieser Ftp-Verbindung
   */
  public String getFtphost() {
    return this.ftphost.toString();
  }

  /**
   * Das Verzeichnis auf dem Ftp-Server, das zur mit Mdl_Connection beschriebenen
   * Verbindung geh&ouml;rt.
   * @param ftpdir Das Verzeichnis auf dem Ftp-Server in der Form (in EBNF:)
   * <code>ftpdir={"/"&lt;verzeichnisname&gt;["/"]}.</code>
   */
  public void setFtpdir(String ftpdir) {
    this.ftpdir.replace(0,this.ftpdir.length(),ftpdir);
  }

  /**
   * Liefert den Namen des Pfades auf dem Ftp-Server; dieser Pfadname ist
   * Bestandteil der mit Mdl_Connection beschriebenen Ftp-Verbindung.
   * @return der Name des Pfades auf dem Ftp-Server dieser Ftp-Verbindung
   */
  public String getFtpdir() {
    return this.ftpdir.toString();
  }

  /**
   * Erzeugt eine String-Repr&auml;sentation dieser Verbindung in der Form
   * ftp://&lt;username&gt;:&lt;passwort&gt;@&lt;ftphost&gt;&lt;ftpdir&gt;
   *
   * @return die ermittelte String-Repr&auml;sentation dieser Verbindung
   */
  public String toString() {
    String usernameString = username.toString();
    String passwordString = password.toString();
    String ftphostString  = ftphost.toString();
    String ftpdirString   = ftpdir.toString();

    String returnString = new String (
        "ftp://" + usernameString
      + ":"      + passwordString
      + "@"      + ftphostString
                 + ftpdirString
    );
    if (returnString.equals("ftp://:@"))
      return "";
    else
      return returnString;
  }

  public Object clone() {
    Mdl_Connection connectionClone =
      new Mdl_Connection(getUsername(),getPassword(),getFtphost(),getFtpdir());
    return connectionClone;
  }
}