package EasyFile;

import java.io.File;
import ipworks.*;
import java.util.*;
import java.lang.reflect.*;
import javax.swing.event.*;
import ipworks.FtpDirListEvent;
import ipworks.FtpEndTransferEvent;
import ipworks.FtpErrorEvent;
import ipworks.FtpPITrailEvent;
import ipworks.FtpStartTransferEvent;
import ipworks.FtpTransferEvent;

/**
 * Verantwortlich f&uuml;r die Bereitstellung aller Ftp-Funktionalit&auml;t,
 * die von EasyFile ben&ouml;tigt wird. Sie vereinfacht die Schnittstelle zur
 * Ftp-Funktionalit&auml;t der Klasse {@link ipworks.Ftp ipworks.Ftp} so weit
 * wie m&ouml;glich zur Verwendung in EasyFile.
 * @author Matthias Ansorg
 * @version 0.3
 */
public class Mdl_Ftp implements FtpEventListener {

  /**
   * Alle zum Aufbau einer Verbindung ben&ouml;tigten Daten. Zus&auml;tzlich
   * werden nur noch die Verbindungseinstellungen (Klasse
   * {@link Mdl_Setting Mdl_Setting}) benouml;tigt.
   */
  public Mdl_Connection connection = new Mdl_Connection();

  Mdl_Hotlist theHotlist;
  Mdl_Log theLog;
  Mdl_Setting theSetting;
  Ftp ipworksFtp = new Ftp();
  File localPath;

  //Fr Debugzwecke kann man sich die gesamte Client-Server-Interaktion in den
  //Logdaten ausgeben lassen:
  boolean writeInteractionLog = true;

  EventListenerList listenerList = new EventListenerList();

  /**
   * Enth&auml;lt die Dateigr&ouml;&szlig;e der Datei, die gerade &uuml;bertragen
   * wird, in Byte. Wenn gerade keine Datei&uuml;bertragung stattfindet, ist der
   * Wert undefiniert.
   */
  int sourceFileSize = 0;

  /**
   * Enth&auml;lt w&auml;hrend einer Datei&uuml;bertragung den zuletzt mit einem
   * ProgressEvent gesendeten Fortschritt der Datei&uuml;bertragung in Prozent.
   * Wird ben&ouml;tigt von der Methode &raquo;public void
   * transfer(FtpTransferEvent e)&laquo;.
   */
  int lastPercent = 0;

  /**
   * Erstellt eine neue Instanz der Ftp-Modellklasse.
   *
   * @param theHotlist anwendungsweit verwendete Liste von gespeicherten
   * Verbindungen zu Ftp-Servern.
   * @param theLog anwendungsweit verwendete Instanz des Datenmodells der
   * Log-Daten
   * @param theSetting anwendungsweit verwendete Grundeinstellungen f&uuml;r
   * Ftp-Verbindungen.
   */
  public Mdl_Ftp(Mdl_Hotlist theHotlist, Mdl_Log theLog, Mdl_Setting theSetting) {
    this.theHotlist = theHotlist;
    this.theLog = theLog;
    this.theSetting = theSetting;
    try {
      ipworksFtp.addFtpEventListener(this);
    }
    catch (TooManyListenersException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Erstellt eine neue Verbindung. Voraussetzung sind passend gesetzte Daten
   * im Feld {@link #connection connection}.
   */
  public void createConnection() {
    try {
      ipworksFtp.setRemoteHost(connection.getFtphost());

      if (! connection.getUsername().equals(""))
        ipworksFtp.setUser(connection.getUsername());
      else
        ipworksFtp.setUser(theSetting.getAnonymUser());

      if (! connection.getPassword().equals(""))
        ipworksFtp.setPassword(connection.getPassword());
      else
        ipworksFtp.setPassword(theSetting.getAnonymPassword());

      if (theSetting.transfermode == theSetting.tmBinary)
        ipworksFtp.setTransferMode(ipworksFtp.tmBinary);
      else
        ipworksFtp.setTransferMode(ipworksFtp.tmASCII);

      if (! theSetting.getFwHost().equals("")) {
        ipworksFtp.setFirewallHost(theSetting.getFwHost());
        ipworksFtp.setFirewallType(theSetting.fwType);
          //^^ ein Hack. Wir nehmen an, dass die verwendeten Konstanten
          //   fr die Firewall-Typen bereinstimmen.
        //Wenn ein sinnvoller Portwert vorhanden, setze entsprechend,
        //ansonsten belassen wir es bei der von setFirewallType gemachten
        //Voreinstellung:
        if (theSetting.fwPort != 0)
          ipworksFtp.setFirewallPort(theSetting.fwPort);
      }
      ipworksFtp.logon();
    }
    catch (IPWorksException exc) {
      theLog.addlogline("Fehler " + exc.getCode() + ": " + exc.getMessage());
    } //Exception bedeutet nicht, dass jetzt keine Verbindung besteht!

    if (ipworksFtp.isConnected()) {
      theLog.addlogline("Verbunden mit " + connection.toString());
      String initDir =
        new String(connection.getFtpdir().equals("")? "/" : connection.getFtpdir());
      try {
        ipworksFtp.setRemotePath(initDir);
        theLog.addlogline("Verzeichniswechsel auf Ftp-Server gelungen in "
                         + ipworksFtp.getRemotePath());
        lsServer();
      }
      catch (IPWorksException exc) {
        theLog.addlogline("Fehler " + exc.getCode() + ": " + exc.getMessage());
        theLog.addlogline("Verzeichniswechsel auf Ftp-Server fehlgeschlagen in " + initDir);
        connection.setFtpdir("");
      }
    }
    else
      theLog.addlogline("Verbindungsaufbau fehlgeschlagen zu " + connection.toString());
  }

  /**
   * Trennt eine bestehende Verbindung vollst&auml;ndig. Wenn keine Verbindung
   * besteht, wird nichts ausgef&uuml;hrt.
   */
  public void removeConnection() {
    try {
      ipworksFtp.logoff();
      theLog.addlogline("Verbindung getrennt mit " + connection.toString());
      connection = new Mdl_Connection("","","","");
      ipworksFtp.setUser("");
      ipworksFtp.setPassword("");
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Wechselt das Verzeichnis auf dem Server.
   * @param directory  Der Name des Unterverzeichnisses, in das gewechselt werden
   * soll. Er darf keine Verzeichnistrenner enthalten, sondern muss stets ein
   * Unterverzeichnis des aktuellen Verzeichnisses auf dem Server bezeichnen.
   * Das kann auch &raquo;..&laquo; als Link auf das &uuml;bergeordnete
   * Verzeichnis sein.
   */
  public void cdServer(String directory) {
    try {
      ipworksFtp.setRemotePath(directory);
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
    //an dieser Stelle kein Verzeichnislisting senden, es muss angefordert werden!
  }

  /**
   * Wechselt das Verzeichnis auf dem lokalen System.
   * @param directory  Der Name des Unterverzeichnisses, in das gewechselt werden
   * soll. Er darf keine Verzeichnistrenner enthalten, sondern muss stets ein
   * Unterverzeichnis des aktuellen Verzeichnisses auf dem lokalen System
   * bezeichnen. Das kann auch &raquo;..&laquo; als Link auf das
   * &uuml;bergeordnete Verzeichnis sein.
   */
  public void cdLocal(String subDir) {
    if (subDir.equals(".."))
      localPath = localPath.getParentFile();
    else
      localPath = new File(localPath, subDir);
    //an dieser Stelle kein Verzeichnislisting senden, es muss angefordert werden!
  }

  /**
   * Erstellt ein Verzeichnis auf dem Server.
   * @param directory  Der Name des Unterverzeichnisses, das
   * erstellt werden soll. Es wird im aktuellen Verzeichnis auf dem
   * Server erstellt; deshalb darf <code>directory</code> keine
   * Verzeichnistrenner enthalten.
   */
  public void mkdirServer(String directory) {
    try {
      ipworksFtp.setRemoteFile(directory);
      ipworksFtp.makeDirectory();
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Erstellt ein Verzeichnis auf dem lokalen System.
   * @param directory  Der Name des Unterverzeichnisses, das
   * erstellt werden soll. Es wird im aktuellen Verzeichnis auf dem
   * lokalen System erstellt; deshalb darf <code>directory</code>
   * keine Verzeichnistrenner enthalten.
   */
  public void mkdirLocal(String subDir) {
    File subDirPath = new File(localPath,subDir);
    boolean madeDir = subDirPath.mkdir();
    if (! madeDir)
      theLog.addlogline("Konnte lokales Verzeichnis "
                        + subDirPath.getPath() + " nicht erstellen");
    //an dieser Stelle kein Verzeichnislisting senden, es muss angefordert werden.
  }

  /**
   * Kopiert die Datei <code>filename</code> aus dem aktuellen
   * Verzeichnis des Ftp-Servers in das aktuelle Verzeichnis auf
   * dem lokalen System.
   * @param filename  Bezeichnet den Namen einer Datei (nicht eines
   * Verzeichnisses!) im aktuellen Verzeichnis auf dem Ftp-Server.
   * Dieser Name darf also keine Verzeichnistrenner enthalten.
   */
  public void download(String filename) {
    try {
      ipworksFtp.setRemoteFile(filename);
      //ToDo: sourceFileSize entspr. Dateigre von RemoteFile setzen.
      ipworksFtp.setLocalFile(new File(localPath,filename).getPath());
        //ndern in Verwendung eines File-Objektes, um Systemunabhngig zu sein.
      ipworksFtp.download();
      //whrend Download und Upload erzeugt ipworksFtp automatisch
      //FtpTransferEvents, die von der Methode void transfer(FtpTransferEvent)
      //dieser Klasse Mdl_Ftp empfangen werden.
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Kopiert die Datei <code>filename</code> aus dem aktuellen
   * Verzeichnis des lokalen Systems in das aktuelle Verzeichnis auf
   * dem Ftp-Server.
   * @param filename  Bezeichnet den Namen einer Datei (nicht eines
   * Verzeichnisses!) im aktuellen Verzeichnis auf dem lokalen System.
   * Dieser Name darf also keine Verzeichnistrenner enthalten.
   */
  public void upload(String filename) {
    try {
      ipworksFtp.setLocalFile(new File(localPath,filename).getPath());
      //ToDo: sourceFileSize entspr. Dateigre von LocalFile setzen.
      ipworksFtp.setRemoteFile(filename);
      ipworksFtp.upload();
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Gibt ein Listing des aktuellen Verzeichnisses auf dem
   * Ftp-Server zur&uuml;ck. Dazu wird ein Strom der Ereignisse
   * {@link StartDirEvent StartDirEvent} (zu Beginn),
   * {@link DirEntryEvent DirEntryEvent} (f&uuml;r jedes Element des Verzeichnisses) und
   * {@link StopDirEvent  StopDirEvent} (am Ende) erzeugt.
   */
  public void lsServer() {
    try {
      ipworksFtp.setRemoteFile("");
        //Filemask zum DirectoryListing; "*" fhrt zu rekursivem Listing
      ipworksFtp.listDirectoryLong();
        //statt listDirectory(), um auch Informationen zu erhalten, ob es sich
        //um ein Verzeichnis handelt.
    }
    catch (Exception exc) {
      theLog.addlogline("Fehler " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  /**
   * Gibt ein Listing des aktuellen Verzeichnisses auf dem
   * lokalen System zur&uuml;ck. Dazu wird ein Strom der Ereignisse
   * {@link StartDirEvent StartDirEvent} (zu Beginn),
   * {@link DirEntryEvent DirEntryEvent} (f&uuml;r jedes Element des Verzeichnisses) und
   * {@link StopDirEvent  StopDirEvent} (am Ende) erzeugt.
   */
  public void lsLocal() {
    File[] fileList;
    if (localPath == null)
      fileList = localPath.listRoots(); //Dateisystemwurzeln listen
    else
      fileList = localPath.listFiles();
    //Eventfolge senden:
    fireStartDirEvent(StartDirEvent.isLocal);
    for (int i=0; i < Array.getLength(fileList); ++i) {
      boolean isDir;
      isDir = fileList[i].isDirectory() ? true : false;
      String fileName = fileList[i].getName();
      if (fileName.equals(""))
        fileName = new String(fileList[i].getPath()); //fr Dateisystemwurzeln
      fireDirEntryEvent(fileName,isDir,DirEntryEvent.isLocal);
    }
    fireStopDirEvent(StopDirEvent.isLocal);
  }

  /**
   * Bricht den Upload oder Download einer Datei ab. Das hei&szlig;t,
   * wenn eine der Methoden {@link #download(String) download(String filename)}
   * oder {@link #download(String) download(String filename)}
   * ausgef&uuml;hrt wird, so wird ihre Ausf&uuml;hrung unterbrochen.
   *
   */
  public void cancelTransfer() {
    try {
      ipworksFtp.abort();
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Bricht eine gerade stattfindende Aktion wie Verzeichniswechsel oder
   * Verzeichnislisting ab, sofern &uuml;berhaupt eine stattfindet.
   */
  public void interrupt() {
    try {
      ipworksFtp.interrupt();
    }
    catch (IPWorksException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void startTransfer(FtpStartTransferEvent e){
    fireStartDirEvent(StartDirEvent.isFtp);
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void dirList(FtpDirListEvent e) {
    fireDirEntryEvent(e.fileName, e.isDir, DirEntryEvent.isFtp);
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void endTransfer(FtpEndTransferEvent e) {
    fireStopDirEvent(StopDirEvent.isFtp);
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void error(FtpErrorEvent e) {
    theLog.addlogline("Fehler Nummer " + e.errorCode + ": " + e.description);
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void PITrail(FtpPITrailEvent e){
    if (! writeInteractionLog) return;

    //Entgegen der Dokumentation von ipworks.FtpPITrailEvent gibt
    //ipworks.FtpPITrailEvent.direction das Ziel der Botschaft an, nicht ihre
    //Herkunft.
    String direction;
    if (e.direction == 0)
      direction =   "From Client: ";
    else
      if (e.direction == 1)
        direction = "From Server: ";
      else // e.direction msste 2 sein
        direction = "For  Info:   ";
    theLog.addlogline("Debug: " + direction + e.message);
  }

  /**
   * Geh&ouml;rt zur Implementierung des Interface ipworks.FtpEventListener
   */
  public void transfer(FtpTransferEvent e) {
    int actualPercent = (e.bytesTransferred * 100) / sourceFileSize;
    if (actualPercent > lastPercent)
      fireProgressEvent(actualPercent);
    if (actualPercent == 100)
      lastPercent = 0;
    else
      lastPercent = actualPercent;
  }

  /**
   * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
   * Erzeugen eines {@link progressEvent progressEvent} benachrichtigen.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param progress der bei der aktuellen Datei&uuml;bertragung bisher erreichte
   * Fortschritt in Prozent. Zul&auml;ssige Werte sind also 0 bis 100.
   */
  void fireProgressEvent(int progress) {
    //die Instanz des Ereignisses erzeugen
    progressEvent e = new progressEvent(this);
    e.progress = progress;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link progressEvent progressEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_FtpEventListener.class) {
        ((Mdl_FtpEventListener)listeners[i+1]).progress(e);
      }
    }
  }

  /**
   * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
   * Erzeugen eines {@link StartDirEvent StartDirEvent} benachrichtigen.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param place Gibt an, ob mit diesem StartDirEvent das Verzeichnislisting
   * eines Ftp-Verzeichnisses (place=={@link StartDirEvent#isFtp StartDirEvent.isFtp})
   * oder eines lokalen VErzeichnisses
   * (place=={@link StartDirEvent#isFtp StartDirEvent.isFtp}) beginnt.
   */
  void fireStartDirEvent(int place) {
    //die Instanz des Ereignisses erzeugen
    StartDirEvent e = new StartDirEvent(this);
    e.place = place;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link StartDirEvent StartDirEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_FtpEventListener.class) {
        ((Mdl_FtpEventListener)listeners[i+1]).StartDir(e);
      }
    }
  }

  /**
   * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
   * Erzeugen eines {@link DirEntryEvent DirEntryEvent} benachrichtigen.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param DirEntry Name des &uuml;bermittelten Verzeichniseintrags, d.h. eines
   * Verzeichnisses oder einer Datei.
   * @param isDir Gibt an, ob der &uuml;bermittelte Verzeichniseintrag ein
   * Verzeichnis ist (isDir==true) oder eine Datei (isDir == false).
   * @param place Gibt an, ob mit diesem DirEntryEvent das Verzeichnislisting
   * eines Ftp-Verzeichnisses (place=={@link DirEntryEvent#isFtp DirEntryEvent.isFtp})
   * oder eines lokalen VErzeichnisses
   * (place=={@link DirEntryEvent#isFtp DirEntryEvent.isFtp}) endet.
   */
  void fireDirEntryEvent(String DirEntry, boolean isDir, int place) {
    //die Instanz des Ereignisses erzeugen
    DirEntryEvent e = new DirEntryEvent(this);
    e.setDirEntry(DirEntry);
    e.isDir = isDir;
    e.place = place;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link DirEntryEvent DirEntryEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_FtpEventListener.class) {
        ((Mdl_FtpEventListener)listeners[i+1]).DirEntry(e);
      }
    }
  }

  /**
   * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
   * Erzeugen eines {@link StopDirEvent StopDirEvent} benachrichtigen.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param place Gibt an, ob mit diesem StopDirEvent das Verzeichnislisting
   * eines Ftp-Verzeichnisses (place=={@link StopDirEvent#isFtp StopDirEvent.isFtp})
   * oder eines lokalen VErzeichnisses
   * (place=={@link StopDirEvent#isFtp StopDirEvent.isFtp}) endet.
   */
  void fireStopDirEvent(int place) {
    //die Instanz des Ereignisses erzeugen
    StopDirEvent e = new StopDirEvent(this);
    e.place = place;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link StopDirEvent StopDirEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_FtpEventListener.class) {
        ((Mdl_FtpEventListener)listeners[i+1]).StopDir(e);
      }
    }
  }

  /**
   * F&uuml;gt einen Mdl_FtpEventListener zur Liste der registrierten Listener
   * hinzu. Das registrierte Objekt empf&auml;ngt ab jetzt alle von
   * Mdl_FtpEventListener empfangbaren Events, d.h. progressEvent,
   * StartDirEvent, DirEntryEvent, StopDirEvent; sofern sie in dieser Instanz
   * von Mdl_Ftp ausgel&ouml;st wurden.
   */
  public void addMdl_FtpEventListener(Mdl_FtpEventListener l) throws TooManyListenersException {
    listenerList.add(Mdl_FtpEventListener.class, l);
  }

  /**
   * Entfernt einen Mdl_FtpEventListener aus der Liste der registrierten
   * Listener.
   */
  public void removeMdl_FtpEventListener(Mdl_FtpEventListener l){
    listenerList.remove(Mdl_FtpEventListener.class, l);
  }
}