package EasyFile;

import java.util.*;
import java.io.*;
import javax.swing.event.*;

/**
 * Verantwortlich f&uuml;r die Verwaltung der Daten der
 * Hotlist.
 * <p>Jeder Eintrag der Hotlist hat eine Identifikationsnummer (ID), mit der
 * dieser Eintrag beim Aufruf von Methoden aus <code>Mdl_Hotlist</code>
 * eindeutig identifiziert wird. Klassen, die diese Hotlist benutzen,
 * m&uuml;ssen sich also neben den Datenfeldern eines Eintrags auch seine ID
 * merken. Der Eintrag mit der ID 0 ist ein leerer Eintrag.</p>
 * <p>Diese Klasse kann au&szlig;erdem die Hotlist in einer Datei auslagern,
 * so dass sie beim n&auml;chsten Start des Programms wieder verf&uuml;gbar
 * ist.</p>
 * @author Matthias Ansorg
 * @version 0.3
 */

public class Mdl_Hotlist {
  EventListenerList listenerList = new EventListenerList();
  Vector thisHotlist = new Vector();

  /**
   * Name der Datei, in den die Daten der Hotlist ausgelagert werden, bevor
   * EasyFile beendet wird.
   */
  final File HotlistFilename = new File("hotlist.txt");
  //ist eine Initialisierung mglich, die fr alle
  //Plattformen funktioniert?

  /**
   * Erzeugen einer neuen Hotlist, in der h&auml;fig benutzte Verbindungen zu
   * Ftp-Servern abgespeichert werden k&ouml;nnen.
   */
  public Mdl_Hotlist() {
    this.addEntry(new Mdl_Connection("","","","")); //der erste Eintrag muss leer sein
    awake();
      //berschreibt ggf. die Wirkung der vorigen Zeile, enthlt aber
      //auch einen leeren ersten Eintrag
  }

  /**
   * Sichert die aktuellen Daten. Eine Methode &raquo;Finalizer&laquo; wird in
   * Java automatisch unmittelbar vor der Zerst&ouml;rung des Objektes
   * aufgerufen.
   */
  protected void finalize () {
    hibernate();
  }

  /**
   * Alle Eintr&auml;ge der Hotlist in eine Datei sichern, deren
   * Name durch das Attribut <code>private final HotlistFilename</code>
   * in dieser Klasse fest einkodiert ist. Diese Methode wird
   * im Finalizer dieser Klasse verwendet, d.h. &uuml;blicherweise bevor die
   * gesamte Anwendung beendet wird.
   */
  void hibernate () {
    //Implementierung fehlt mangels Programmierer
  }

  /**
   * Die gesicherte Hotlist wieder laden. Wenn die Datei zum Sichern der Hotlist
   * nicht existiert, ist die &raquo;geladene&laquo; Hotlist leer. Der Dateiname
   * der Datei, in der die Hotlist gesichert wird, ist durch das Attribut
   * <code>private final HotlistFilename</code> in dieser Klasse fest einkodiert.
   * Diese Methode wird im Konstruktor dieser Klasse verwendet, d.h.
   * &uuml;blicherweise beim Start der gesamten Anwendung.
   */
  void awake () {
    //Implementierung fehlt mangels Programmierer
  }

  /**
   * F&uuml;gt einen Eintrag zur Hotlist hinzu. Es wird das
   * Ereignis {@link addedEntryEvent addedEntryEvent} erzeugt,
   * das die anzeigenden Klassen empfangen und daraufhin
   * den neuen Eintrag anzeigen k&ouml;nnen.
   * @param  e  Der Eintrag, der zur Hotlist hinzugef&uuml;gt
   * werden soll.
   */
  public void addEntry(Mdl_Connection e) {
    Mdl_Connection eClone = (Mdl_Connection)e.clone();
    thisHotlist.add(eClone);
      //keine Referenz hinzufgen, denn sonst zeigen alle Eintrge auf
      //theFtp.connection, sind also identisch. Deshalb e.clone()
    int connectionID = thisHotlist.size()-1;
    fireAddedEntryEvent(connectionID);
  }

/**
 * Daten eines Eintrags der Hotlist erhalten.
 * @param id  Die Identifikationsnummer des Eintrags der Hotlist,
 * dessen Daten man erhalten will.
 */
  public Mdl_Connection getEntry(int id) {
    return (Mdl_Connection)thisHotlist.get(id);
  }

  /**
   * Einen Eintrag aus der Hotlist entfernen.
   *
   * @param id  Die Identifikationsnummer des Eintrags der Hotlist,
   * der entfernt werden soll.
   */
  public void removeEntry(int id) {
    thisHotlist.set(id,null);
    fireRemovedEntryEvent(id);
  }

  /**
   * Die Anzahl der Eintr&auml;ge der Hotlist ermitteln
   *
   * @return die ermittelte Anzahl der Eintr&auml;ge in dieser Hotlist.
   */
  public int getNumberOfEntrys() {
    return thisHotlist.size();
  }

  /**
   * Ermittelt die ID eines Elements der Hotlist.
   *
   * @param anObject das Objekt in der Hotlist, dessen ID ermittelt werden soll.
   * @return die ermittelte ID des &uuml;bergebenen Objektes. Sie liegt im
   * Bereich von 0 bis getNumberOfEntrys(); der Wert -1 bedeutet, dass das
   * &uuml;bergebene Objekt nicht gefunden wurde.
   * @see Vector#indexOf(Object elem)
   */
  public int indexOf(Object anObject) {
    return thisHotlist.indexOf(anObject);
  }

  /**
   * F&uuml;gt ein Element mit einer vorgegebenen ID ein. Hat bereits ein
   * Element diese ID, so wird es mit den Daten des neuen Elements
   * &uuml;berschrieben. Der Wertebereich der ID darf von 0 bis
   * getNumberOfEntrys() reichen; im letzteren Fall wird ein Element
   * angef&uuml;gt.
   * @param anObject das einzuf&uuml;gende Objekt; es muss vom Typ
   * Mdl_Connection sein.
   * @param id die Identifikationsnummer (ID), die das neu hinzugef&uuml;gte
   * Element haben soll.
   */
  public void insertElementAt(Object anObject,int id) {
    if (id >= 0 && id < getNumberOfEntrys())
      thisHotlist.set(id,anObject);
    else if (id == getNumberOfEntrys())
      thisHotlist.add(anObject);
  }

  /**
   * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
   * Erzeugen eines {@link addedEntryEvent addedEntryEvent} benachrichtigen.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param id Die Identifikationsnummer des hinzugekommenen Eintrags.
   */
  void fireAddedEntryEvent(int id) {
    //die Instanz des Ereignisses erzeugen
    addedEntryEvent e = new addedEntryEvent(this);
    e.id = id;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link addedEntryEvent addedEntryEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_HotlistEventListener.class) {
        ((Mdl_HotlistEventListener)listeners[i+1]).addedEntry(e);
      }
    }
  }

/**
 * Alle Mdl_HotlistEventListener, die sich registriert haben, durch das
 * Erzeugen eines {@link removedEntryEvent removedEntryEvent} benachrichtigen.
 * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
 * zu Klasse javax.swing.event.EventListenerList, enthalten in der
 * API-Dokumentation zu Java SDK 1.3.
 *
 * @param id Die Identifikationsnummer des gel&ouml;schten Eintrags.
 */
  void fireRemovedEntryEvent(int id) {
    //die Instanz des Ereignisses erzeugen
    removedEntryEvent e = new removedEntryEvent(this);
    e.id = id;
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link addedEntryEvent addedEntryEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_HotlistEventListener.class) {
        ((Mdl_HotlistEventListener)listeners[i+1]).removedEntry(e);
      }
    }
  }

/**
 * Hinzuf&uuml;gen eines Mdl_HotlistEventListener. Diese Implementierung richtet
 * sich nach den Hinweisen in der Dokumentation zu Klasse
 * javax.swing.event.EventListenerList, enthalten in der API-Dokumentation zu
 * Java SDK 1.3.
 * @param l Der Mdl_HotlistEventListener, der hinzugef&uuml;gt werden soll.
 */
  public void addMdl_HotlistEventListener(Mdl_HotlistEventListener l) throws TooManyListenersException {
    listenerList.add(Mdl_HotlistEventListener.class, l);
  }

/**
 * Entfernen eines Mdl_HotlistEventListener. Diese Implementierung richtet sich
 * nach den Hinweisen in der Dokumentation zu Klasse javax.swing.event.EventListenerList,
 * enthalten in der API-Dokumentation zu Java SDK 1.3.
 * @param l Der Mdl_HotlistEventListener, der entfernt werden soll.
 */
  public void removeMdl_HotlistEventListener(Mdl_HotlistEventListener l){
    listenerList.remove(Mdl_HotlistEventListener.class, l);
  }
}