package EasyFile;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * Eine Umschlagklasse, die ein Objekt der Klasse Mdl_Hotlist so verpackt, dass
 * daraus eine Modellklasse fr JComboBoxen wird, deren Eintrge die Elemente
 * der Hotlist sind. Wenn in einer GUI eine ComboBox mit Eintr&auml;gen einer
 * Hotlist verwendet wird, kann und soll ausschlie&szlig;lich diese Klasse als
 * Datenmodell verwendet werden, direkte Zugriffe auf das darunterliegende
 * Objekt der Klasse Mdl_Hotlist sind zu vermeiden.<br>
 * JComboBoxen, die Mdl_Hotlistbox als Datenmodell verwenden, mssen keinen
 * eigenen ListCellRenderer implementieren, trotz dass die Objekte in der
 * JComboBox keine Strings sind. Der DefaultRenderer ruft automatisch die
 * Methode &raquo;toString&laquo; dieser Objekte auf.
 */
public class Mdl_Hotlistbox extends AbstractListModel implements MutableComboBoxModel, Mdl_HotlistEventListener {

  Mdl_Hotlist theHotlist;
  Object selectedItem; //selektierte Zeile der JComboBox

  /**
   * Erzeugen eines Mdl_Hotlistbox.
   *
   * @param theHotlist Hotlist, deren Verbindungseintrgen die Elemente dieser
   * Mdl_Hotlistbox sein sollen. Das wird realisiert, indem einfach die
   * entsprechenden Funktionen von theHotlist aufgerufen werden.
   */
  public Mdl_Hotlistbox(Mdl_Hotlist theHotlist) {
    this.theHotlist = theHotlist;
    try {
      this.theHotlist.addMdl_HotlistEventListener(this);
      //post: diese Mdl_Hotlistbox enthlt alle Elemente von theHotlist
    }
    catch (TooManyListenersException exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Gibt das Element an der Position index zur&uuml;ck. Der Index, unter dem ein
   * Element in dieser Klasse und der Klasse {@link Mdl_Hotlist Mdl_Hotlist}
   * angesprochen wird, ist stets identisch. {@link Mdl_Hotlistbox Mdl_Hotlistbox}
   * enthlt hier Objekte vom Typ {@link Mdl_Connection Mdl_Connection}.
   * JComboBoxen, die Mdl_Hotlistbox als Datenmodell verwenden, mssen keinen
   * eigenen ListCellRenderer implementieren, trotz dass die Objekte in der
   * JComboBox keine Strings sind. Der DefaultRenderer ruft automatisch die
   * Methode &raquo;toString&laquo; dieser Objekte auf.
   * Diese Methode geh&ouml;rt zur Implementierung des Interface
   * {@link ComboBoxModel ComboBoxModel}.
   *
   * @param index Position des angeforderten Elements. M&ouml;gliche Werte sind
   * 0 bis {@link #getSize() getSize()}-1. Liegt der Wert nicht in diesen
   * Grenzen, wird null zur&uuml;ckgegeben.
   */
  public Object getElementAt(int index) {
    if (index >= 0 && index < theHotlist.getNumberOfEntrys()) {
      return theHotlist.getEntry(index);
    }
    else
      return null;
  }

  /**
   * Liefert die Position (den Index) des &uuml;bergebenen Objektes in der
   * Hotlist. Die Indizes in Mdl_Hotlistbox und dem darin verwendeten Objekt
   * von {@link Mdl_Hotlist Mdl_Hotlist} sind jeweils identisch.
   *
   * @param anObject das Objekt, dessen Index in der Liste ermittelt werden soll.
   * @return der Index in der Liste. Wertebereich von 0 bis getSize()-1. Wenn
   * das &uuml;bergebene Objekt nicht gefunden wurde, wird -1
   * zur&uuml;ckgegeben.
   * @see Vector#indexOf(Object elem)
   */
  public int getIndexOf(Object anObject) {
      return theHotlist.indexOf(anObject);
  }

  //implementiert javax.swing.MutableComboBoxModel
  public void addElement(Object anObject) {
    theHotlist.addEntry((Mdl_Connection)anObject);
    fireIntervalAdded(this,getSize()-1, getSize()-1);
    if ( getSize() == 1 && getSelectedItem() == null && anObject != null ) {
      setSelectedItem( anObject );
    }
  }

  /**
   * Gibt die Anzahl der Elemente in diesem Objekt von Mdl_Hotlistbox zurck.
   * Methode geh&ouml;rt zur Implementierung des Interface
   * {@link ComboBoxModel ComboBoxModel}.
   */
  public int getSize() {
    return theHotlist.getNumberOfEntrys();
  }

  /**
   * Zwischengespeichertes selektiertes Objekt der JComboBox zur&uuml;ckgeben.
   * Es ist hier stets vom Typ Mdl_Connection, aber weil diese Methode zur
   * Implementierung des Interface {@link ComboBoxModel ComboBoxModel}
   * geh&ouml;rt, kann hier nur allgemein Object als Typ des R&uuml;ckgabewertes
   * angegeben werden.
   */
  public Object getSelectedItem() {
    return selectedItem;
  }

  /**
   * Setze eine Zeile der JComboBox als die selektierte Zeile fest. Diese
   * Methode geh&ouml;rt zur Implementierung des Interface
   * {@link ComboBoxModel ComboBoxModel}. anItem ist hier stets vom Typ
   * Mdl_Connection, jedoch schreibt das Interface
   * {@link ComboBoxModel ComboBoxModel} den Typ Object vor.
   *
   * @param anItem der aktuell selektierte Eintrag der JComboBox
   */
  public void setSelectedItem(Object anItem) {
    if ( (selectedItem != null && !selectedItem.equals( anItem )) ||
         selectedItem == null && anItem != null ) {
      selectedItem = anItem;
      fireContentsChanged(this, -1, -1);
    }
  }

  /**
   * F&uuml;gt ein Element an einer definierten Position ein. Dabei wird das
   * ggf. vorher dort befindliche Element &uuml;berschrieben. Implementierung
   * des Interface javax.swing.MutableComboBoxModel.
   */
  public void insertElementAt(Object anObject,int index) {
    theHotlist.insertElementAt(anObject,index);
    fireIntervalAdded(this, index, index);
  }

  // implements javax.swing.MutableComboBoxModel
  public void removeElementAt(int index) {
    if ( getElementAt( index ) == getSelectedItem() ) {
      if ( index == 0 ) {
        setSelectedItem( getSize() == 1 ? null : getElementAt( index + 1 ) );
      }
      else {
        setSelectedItem( getElementAt( index - 1 ) );
      }
    }
    theHotlist.removeEntry(index);
    fireIntervalRemoved(this, index, index);
  }

  // implements javax.swing.MutableComboBoxModel
  public void removeElement(Object anObject) {
    int index = theHotlist.indexOf(anObject);
    if ( index != -1 ) {
      removeElementAt(index);
    }
  }

  /**
   * Reaktion darauf, dass ein neuer Eintrag zur anwendungsweit verwendeten
   * Hotlist (Variable theHotlist) hinzugekommen ist.
   *
   * @param e  Event, das die Identifikationsnummer des neuen Hotlist-Eintrags
   * enth&auml;lt.
   */
  public void addedEntry(addedEntryEvent e){
    /*
     * Hier nichts ausfhren, weil jede Funktion dieser Klasse, die theHotlist
     * verndert, selbst fireIntervalAdded usw. ausfhrt. Das macht die
     * Ereignisbehandlung bersichtlicher, statt auf ein Ereignis zu warten
     * um ein Ereignis weiterzuschicken.
     */
  }

  /**
   * Reaktion darauf, dass ein Eintrag der anwendungsweit verwendeten Hotlist
   * (Variable theHotlist) gel&ouml;scht wurde.
   *
   * @param e  Event, das die Identifikationsnummer des gel&ouml;schten
   * Hotlist-Eintrags enth&auml;lt.
   */
  public void removedEntry(removedEntryEvent e){
    /*
     * Hier nichts ausfhren, weil jede Funktion dieser Klasse, die theHotlist
     * verndert, selbst fireIntervalAdded usw. ausfhrt. Das macht die
     * Ereignisbehandlung bersichtlicher, statt auf ein Ereignis zu warten
     * um ein Ereignis weiterzuschicken.
     */
  }
}