package EasyFile;

import java.util.*;
import javax.swing.event.*;

/**
 * Verantwortlich f&uuml;r die Verwaltung der Log-Daten, die
 * von verschiedenen anderen Klassen kommen k&ouml;nnen. Die
 * Log-Daten werden vom Starten bis zum Beenden des Programms
 * gesammelt, aber nicht in einer Datei gespeichert.
 * @author Henrik Warnecke, J&ouml;rg Hoos, Matthias Ansorg, Thomas Martin, Ralf Mei&szlig;ner
 * @version 0.3
 */

public class Mdl_Log {
  EventListenerList listenerList = new EventListenerList();

  /**
   * Konstruiert ein neues Datenmodell zur Verwaltung der LogDaten einer
   * Anwendung.
   */
  public Mdl_Log() {
  }

  /**
   * F&uuml;gt eine Zeile zu den Log-Daten hinzu. Dazu werden alle
   * interessierten Objekte durch das Erzeugen eines Ereignisses, das diese
   * Log-Daten enth&auml;lt, benachrichtigt.
   *
   * @param s die zur Anzeige der LogDaten hinzuzuf&uuml;gende Zeile.
   */
  public void addlogline(String s) {
    fireLogEvent(s);
  }

  /**
   * Ein LogEvent erzeugen, das alle reistrierten Mdl_LogEventListener
   * benachrichtigt.
   * Diese Implementierung richtet sich nach den Hinweisen in der Dokumentation
   * zu Klasse javax.swing.event.EventListenerList, enthalten in der
   * API-Dokumentation zu Java SDK 1.3.
   *
   * @param s die Log-Daten, die in dem zu erzeugenden LogEvent enthalten sein
   * sollen. Es soll sich um eine Zeile Text handeln.
   */
  void fireLogEvent(String s) {
    //die Instanz des Ereignisses erzeugen
    LogEvent e = new LogEvent(this);
    e.setLogMsg(s);
    //EventListenerList garantiert ein Array ungleich null:
    //Die Elemente des Arrays geben Typ und Instanz der Listener an.
    Object[] listeners = listenerList.getListenerList();

    //Alle Listener vom letzten bis zum ersten durchgehen und die
    //an {@link LogEvent LogEvents} interessierten benachrichtigen.
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==Mdl_LogEventListener.class) {
        ((Mdl_LogEventListener)listeners[i+1]).logline(e);
      }
    }
  }

  public void addMdl_LogEventListener(Mdl_LogEventListener l) throws TooManyListenersException {
    listenerList.add(Mdl_LogEventListener.class, l);
  }

  public void removeMdl_LogEventListener(Mdl_LogEventListener l){
    listenerList.remove(Mdl_HotlistEventListener.class, l);
  }
}