package EasyFile;

/**
 * Verantwortlich f&uuml;r die Speicherung der
 * Ftp-&Uuml;bertragungseinstellungen.
 * @author Henrik Warnecke, J&ouml;rg Hoos, Matthias Ansorg, Thomas Martin, Ralf Mei&szlig;ner
 * @version 0.3
 */

public class Mdl_Setting {

  public static final int tmDefault=0;
  public static final int tmASCII=1;
  public static final int tmBinary=2;

/**
 * Einstellung der &Uuml;bertragungsart zwischen Ftp-Server
 * und Ftp-Client. Der Wert kann numerisch oder unter Verwendung
 * entsprechender Konstanten gesetzt werden:
 * <UL>
 *   <LI><code>tmDefault</code> = 0. Keine &Auml;nderung der
 *   &Uuml;bertragungsart, es wird die &Uuml;bertragungsart
 *   beibehalten, die nach &Ouml;ffnen der Ftp-Verbindung
 *   besteht.</LI>
 *   <LI><code>tmASCII</code> = 1. &Uuml;brtragungsart auf
 *   ASCII setzen.</LI>
 *   <LI><code>tmBinary</code> = 2 &Uuml;bertragungsart auf
 *   Bin&auml;r setzen.</LI>
 * </UL>
 */
  public int transfermode;

  public static final int fwNone=0;
  public static final int fwTunnel=1;
  public static final int fwSOCKS4=2;
  public static final int fwSOCKS5=3;

  /**
   * Einstellung des Firewall-Typs. Erm&ouml;glicht gleichzeitig
   * eine Ftp-Verbindung &uuml;ber einen Proxy-Server. Der Wert
   * kann numerisch oder unter Verwendung entsprechender
   * Konstanten gesetzt werden:
   * <UL>
   *   <LI><code>fwNone</code> = 0. Keinen Firewall verwenden. Dies
   *   ist die Voreinstellung.</LI>
   *   <LI><code>fwTunnel</code> = 1. Tunneling Proxy als Firewall
   *   verwenden.</LI>
   *   <LI><code>fwSOCKS4</code> = 2 SOCKS4 Proxy als Firewall
   *   verwenden.</LI>
   *   <LI><code>fwSOCKS5</code> = 3 SOCKS5 Proxy als Firewall
   *   verwenden.</LI>
   * </UL>
   */
  public int fwType = fwNone;

/**
 * Nummer des zu verwendenen Ports auf dem eingestellten
 * Firewall-Host.
 *
 * @see #getFwHost
 * @see #setFwHost(String fwHost)
 */
  public int fwPort;

/**
 * Namens des zu verwendenden Firewall-Hosts.
 */
  StringBuffer fwHost = new StringBuffer();

/**
 * Benutzername zum Einloggen bei anonymem Login auf einem Ftp-Server.
 */
  StringBuffer anonymUser = new StringBuffer();

/**
 * Passwort, das zusammen mit dem Benutzernamen {@link #anonymUser anonymUser}
 * zum anonymen Login auf einem Ftp-Server verwendet wird.
 */
  StringBuffer anonymPassword = new StringBuffer();

  /**
   * Standardkonstruktor ohne weitere T&auml;tigkeiten.
   */
  public Mdl_Setting() {
  }

  /**
   * Den Namen des zu verwendenden Firewall-Hosts setzen.
   *
   * @param fwHost der Name des zu setzenden Firewall-Hosts
   */
  public void setFwHost(String fwHost) {
    this.fwHost.replace(0,this.fwHost.length(),fwHost);
  }

  /**
   * Liefert den Namen des zu verwendenden Firewall-Hosts in der Form
   * subdomain.hostname.top-level-domain
   *
   * @return der Name des zu verwendenden Firewall-Hosts
   */
  public String getFwHost() {
    return fwHost.toString();
  }

  /**
   * Den zu verwendenden Benutzernamens f&uuml;r anonymes Login setzen.
   *
   * @param anonymUser der Benutzername f&uuml;r anonymes Login
   */
  public void setAnonymUser(String anonymUser) {
    this.anonymUser.replace(0,this.anonymUser.length(),anonymUser);
  }

  /**
   * Liefert den f&uuml;r anonymes Login verwendeten Benutzernamen
   *
   * @return der Benutzername f&uuml;r anonymes Login
   */
  public String getAnonymUser() {
    return anonymUser.toString();
  }

  /**
   * Das zu verwendende Passwort f&uuml;r anonymes Login setzen.
   *
   * @param anonymPassword das Passwort f&uuml;r anonymes Login.
   */
  public void setAnonymPassword(String anonymPassword) {
    this.anonymPassword.replace(0,this.anonymPassword.length(),anonymPassword);
  }

  /**
   * Liefert das f&uuml;r anonymes Login verwendete Passwort
   *
   * @return das Passwort f&uuml;r anonymes Login
   */
  public String getAnonymPassword() {
    return anonymPassword.toString();
  }

}